////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

//create an image for sending error information
document.writeln('<img id="xopus_errorhandler_logger" src="http://xopus.org/media/nix.gif" width="1" height="1" style="position:absolute; top:-100px; left:-100px; display:none; visibility:hidden;" />');

function ErrorHandler()
{
  this.locale = null;
  
  this.errorMessage            = "errorMessage";
  this.warningMessage          = "warningMessage";
  this.recoverableErrorMessage = "recoverableErrorMessage";
  this.logger = document.getElementById('xopus_errorhandler_logger');
  
  this.FATAL_ERROR = 42;
}

ErrorHandler.prototype =
{
  /**
   * Set the locale.
   */
  setLocale: function (locale)
  {
    this.locale = locale;
  },
  
  /**
   * See if we have a locale.
   */
  hasLocale: function ()
  {
    return this.locale != null;
  },
  
  /**
   * Get a localized string.
   */
  getLocalizedString: function (id, hash)
  {
    if (this.hasLocale())
      return this.locale.getString(id, hash);
    else
      return this.getStringFallback(id, hash);
  },
  
  /**
   * There is no locale yet, try to show some message.
   */
  getStringFallback: function (id, hash)
  {
    var err = "An error occured during initialization of the Xopus localization engine.\n";
    err += "Error id: " + id + "\n";
    err += "Error properties:\n";
    
    for (var prop in hash)
      err += "  " + prop + ": " + hash[prop] + "\n";
    
    return err;
  },
  
  /**
   * Show a fatal error.
   */
  showFatalError: function (id, hash)
  {
    var msg = {msg: this.getLocalizedString(id, hash)};
    var errorMsg = this.getLocalizedString(this.errorMessage, msg);

    //store error if needed
    if (xopus_globs.logerrors == true) {
      this.logger.src = 'http://xopus.org/logerrors.jsp?error=' + escape(errorMsg) + '&type=fatal&appVersion=' + escape(navigator.appVersion) + '&time=' + escape(new Date());
      __xopus_alert('logging:' + this.logger.src);
    }
    
    __xopus_alert(errorMsg);
    throw new XopusError(msg.msg, this);
  },
  
  /**
   * Show a warning.
   */
  showWarning: function (id, hash)
  {
    var msg = {msg: this.getLocalizedString(id, hash)};
    var errorMsg = this.getLocalizedString(this.warningMessage, msg);

    //store error if needed
    if (xopus_globs.logerrors == true) {
      this.logger.src = 'http://xopus.org/logerrors.jsp?error=' + escape(errorMsg) + '&type=warning&appVersion=' + escape(navigator.appVersion) + '&time=' + escape(new Date());
      __xopus_alert('logging:' + this.logger.src);
    }
    
    __xopus_alert(errorMsg);
  },
   
  /**
   * Show a recoverable error.
   */
  showRecoverableError: function (id, hash)
  {
    var msg = {msg: this.getLocalizedString(id, hash)};
    var errorMsg = this.getLocalizedString(this.recoverableErrorMessage, msg);

    //store error if needed
    if (xopus_globs.logerrors == true) {
      this.logger.src = 'http://xopus.org/logerrors.jsp?error=' + escape(errorMsg) + '&type=recoverable&appVersion=' + escape(navigator.appVersion) + '&time=' + escape(new Date());
    }
    
    if (!confirm(errorMsg))
      throw new XopusError(msg.msg, this);
  },
  
  /**
   * See if the given DOM has a parse error.
   */
  hasDOMError: function (dom)
  {
    return dom == null || dom.documentElement == null || dom.documentElement.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml";
  },
  
  /**
   * Extract the error information from the given DOM.
   */
  getDOMError: function (dom, uri)
  {
    var hash = {};
    
    hash.line    = "";
    hash.col     = "";
    hash.reason  = "";
    hash.srcText = "";
    hash.uri     = uri;
    
    if (dom && dom.parseError)
    {
      hash.line    = dom.parseError.line;
      hash.col     = dom.parseError.linepos;
      hash.reason  = dom.parseError.reason;
      hash.srcText = dom.parseError.srcText;
    }
    else if (dom && dom.documentElement && dom.documentElement.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml")
    {
      hash.reason  = dom.text;
    }
          
    return hash;  
  }
};

function XopusError(reason, errorHandler)
{
  this.reason = reason;
  this.errorHandler = errorHandler;
}

XopusError.prototype = 
{
  /**
   * Get the reason for this error.
   */
  getReason: function (msgID)
  {
    return this.errorHandler.getLocalizedString(msgID, {reason: this.reason});
  }
};

//setTimeout('xopus_globs.errorHandler.showWarning("viewHasNoActions", {uri:"skljsl", viewID:1});', 5000);
numScriptsLoading--;
