////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

function Locale() {
  this.language = null;
  this.defaultMsgs = null;
  this.msgs = {};

  // Check language
  if (!xopus_globs.languages[xopus_globs.language])
    xopus_globs.store('language', xopus_globs.defaultLanguage);

  //load default messages
  var uri = xopus_globs.basePath + 'xopus/locale/xopusMessages_default.xml';
  var xml = xopus_globs.xmllib.load(uri, true);
  var nodes = xml.selectNodes('/messages/m');
  this.defaultMsgs = {};
  for (var i=0; i<nodes.length; i++) {
    var ids = nodes[i].getAttribute('id').split(',');
    var msg = xopus_globs.utils.getChildren(nodes[i]).xml.replace(/\<\!\[CDATA\[/g, '').replace(/\]\]\>/g, '');
    for (var j=0; j<ids.length; j++) {
      this.defaultMsgs[ids[j]] = msg;
    }
  }
  
  // Load file
  this.setLanguage(xopus_globs.language);
}

Locale.prototype={
  /**
   * Set the current language.
   */
  setLanguage: function (code)
  {
    if (code == this.language) return;
    this.language = code;
    
    if (!this.msgs[code]) {
      var msgs = {};

      //copy default messages
      for (var id in this.defaultMsgs) {
        msgs[id] = this.defaultMsgs[id];
      }
  
      var uri = xopus_globs.basePath + 'xopus/locale/xopusMessages_' + code + '.xml';
      var xml = xopus_globs.xmllib.load(uri, true);
      if (!xml) {
        if (code.indexOf("-") == 2) {
          code = code.substring(0, 2);
          this.language = code;
          uri = xopus_globs.basePath + 'xopus/locale/xopusMessages_' + code + '.xml';
          xml = xopus_globs.xmllib.load(uri, true);
        }
      }
      if (!xml) {
        xopus_globs.errorHandler.showRecoverableError("loadError", {uri:uri, text:"could not load locale file"});

        code = xopus_globs.defaultLanguage;
        this.language = code;
        uri = xopus_globs.basePath + 'xopus/locale/xopusMessages_' + code + '.xml';
        xml = xopus_globs.xmllib.load(uri, true);
      }
      if (!xml) return null;

      var nodes = xml.selectNodes('/messages/m');
      for (var i=0; i<nodes.length; i++) {
        var ids = nodes[i].getAttribute('id').split(',');
        var msg = xopus_globs.utils.getChildren(nodes[i]).xml.replace(/\<\!\[CDATA\[/g, '').replace(/\]\]\>/g, '');
        for (var j=0; j<ids.length; j++) {
          msgs[ids[j]] = msg;
        }
      }

      //add languages from xopus_globs (bit dirty, but so what)
      for (var langCode in xopus_globs.languages) msgs[langCode] = xopus_globs.languages[langCode];

      //load CMS specific messages
      var uri = xopus_globs.basePath + 'xopusPlugins/xopusMessages_' + code + xopus_globs.cmsPostfix + '.xml';
      var xml = xopus_globs.xmllib.load(uri, true);
      if (xml) {
        var nodes = xml.selectNodes('/messages/m');
        for (var i=0; i<nodes.length; i++) {
          var ids = nodes[i].getAttribute('id').split(',');
          var msg = xopus_globs.utils.getChildren(nodes[i]).xml.replace(/\<\!\[CDATA\[/g, '').replace(/\]\]\>/g, '');
          for (var j=0; j<ids.length; j++) {
            msgs[ids[j]] = msg;
          }
        }
      }

      this.msgs[code] = msgs;

      window.status = this.getString('done');
    }

    xopus_globs.store('language', code);
  },
  
  /**
   * Get a list of options for the current language. The first item is the preferred option.
   */
  getLanguages: function()
  {
    var langs = [];
    langs.push(this.language);
    
    var idx = this.language.indexOf("-");
    if (idx != -1)
      langs.push(this.language.substr(0, idx));
      
    if (langs[0] != xopus_globs.defaultLanguage && (!langs[1] || langs[1] != xopus_globs.defaultLanguage))
      langs.push(xopus_globs.defaultLanguage);
    
    if (langs[0] != "en" && (!langs[1] || langs[1] != "en") && (!langs[2] || langs[2] != "en"))
      langs.push("en");
    
    return langs;
  },
  
  /**
   * Get a string for the given id in the current language.
   *
   * hash is a mapping for variable names to their values.
   *
   * Example call: locale.getString("moveUp", {nodeName: "book"})
   */
  getString: function (id, hash)
  {
    var string = null;
    var msgs = this.msgs[this.language];
    if (!msgs) msgs = this.msgs[xopus_globs.defaultLanguage];
    if (msgs) string = msgs[id];
    if (!string) {
      string = id;
      var pos = string.indexOf(':');
      if (pos != -1) string = string.substring(pos + 1);
    }

    if (string.indexOf("$") != -1) {
      //add some standard variables to the hash
      if (!hash) hash = {};
      hash['xopus_globs.basePath'] = xopus_globs.basePath;
    
      //replace hashed name value pairs
      // Make an array of all hashable names
      var names = [];
      for (var name in hash) names[names.length] = name;

      // Sort the names
      names = names.sort(); 

      // Loop names backwards to guarantee longest matches first
      for (var i = names.length - 1; i >= 0; i--) {
        var name = names[i];
        var value = hash[name];
        if (typeof(value) == "string" || typeof(value) == "number")
          string = string.replace("$" + name, value);
      }

      // Restore enters
      string = string.replace(/\\n/g, "\n");
    }

    return string;
  }
};

numScriptsLoading--;
