////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

var SOMITEM_SCHEMA         = 4 * 1024;
var SOMITEM_ATTRIBUTE      = SOMITEM_SCHEMA + 1;
var SOMITEM_ATTRIBUTEGROUP = SOMITEM_SCHEMA + 2;
var SOMITEM_NOTATION       = SOMITEM_SCHEMA + 3;

var SOMITEM_ANYTYPE        = 8 * 1024;
var SOMITEM_DATATYPE       = SOMITEM_ANYTYPE  + 256;
var SOMITEM_SIMPLETYPE     = SOMITEM_DATATYPE + 256;
var SOMITEM_COMPLEXTYPE    = 9 * 1024;

var SOMITEM_PARTICLE       = 16 * 1024;
var SOMITEM_ANY            = SOMITEM_PARTICLE + 1;
var SOMITEM_ANYATTRIBUTE   = SOMITEM_PARTICLE + 2;
var SOMITEM_ELEMENT        = SOMITEM_PARTICLE + 3;
var SOMITEM_GROUP          = SOMITEM_PARTICLE + 256;
  
var SOMITEM_ALL            = SOMITEM_GROUP + 1;
var SOMITEM_CHOICE         = SOMITEM_GROUP + 2;
var SOMITEM_SEQUENCE       = SOMITEM_GROUP + 3;
var SOMITEM_EMPTYPARTICLE  = SOMITEM_GROUP + 4;

// Attribute uses
var SCHEMAUSE_OPTIONAL     = 0;
var SCHEMAUSE_PROHIBITED   = 1;
var SCHEMAUSE_REQUIRED     = 2;

var SCHEMACONTENTTYPE_EMPTY       = 0;
var SCHEMACONTENTTYPE_TEXTONLY    = 1;
var SCHEMACONTENTTYPE_ELEMENTONLY = 2;
var SCHEMACONTENTTYPE_MIXED       = 3;

// Schema extension return values
var SE_CONTINUE = 0;
var SE_STOP     = 1;
var SE_CANCEL   = 2;

function SchemaHandler (xmlhandler)
{
  this.cache = xopus_globs.objectFactory.createXMLSchemaCache();
  this.xmlhandler = xmlhandler;
}

SchemaHandler.prototype =
{
  /**
   * Loads a schema from the given URI.
   */
  loadSchema: function (schemaURI, theNamespace) 
  {
    var theXsd = xopus_globs.datadriver.loadSchema(schemaURI);

    if (theNamespace == null)
      theNamespace = this.getTargetNamespace(theXsd);

    // Apply IE bug workaround
    this.fixGroups(theXsd);
          
    this.resolveIncludes(theXsd, schemaURI); 
    this.loadImports(theXsd, schemaURI);
    
    try
    {
      this.cache.add(theNamespace, theXsd);
      
      // Dump the schema if requested
      if (dumpSchema) 
        xopus_globs.utils.debug(theXsd.xml);
    }
    catch (e)
    {
      var props = e.description.split("\n");
      var hash = {uri: schemaURI, xpath: (props[1] ? props[0] : "unknown"), reason: (props[1] ? props[1] : props[0])};
      
      xopus_globs.errorHandler.showRecoverableError("schemaError", hash);
      
      // Retry loading
      this.loadSchema(schemaURI, theNamespace);
    }
  },

  /**
   * Attach the SchemaHandler to the given xml document.
   */
  attachTo: function (xmlDoc)
  {
    this.theDoc = xmlDoc;
    
    if (!xopus_globs.useXopusSOM)
      this.theDoc.schemas = this.cache;
  
    this.validator = new SchemaValidator(this.getSchemaCache());
  },
  
  /**
   * Get the schema cache.
   */
  getSchemaCache: function()
  {
    if (!xopus_globs.useXopusSOM)
      return this.theDoc.namespaces;
    else
      return this.cache;
  },
  
  /**
   * Get the target namespace for the given schema.
   */
  getTargetNamespace: function (theXsd)
  {
    var theNamespace = theXsd.documentElement.getAttribute("targetNamespace");

    if (theNamespace == null)
     return "";
    else
     return theNamespace;
  },
  
  /**
   * Checks if a node with the given name can be created in the given parent node.
   */
  canContainNode: function (parentNode, nodeName, namespaceURI)
  {
    var possibleChildren = this.getPossibleChildren(parentNode);
    
    var child = possibleChildren[nodeName];
    
    return child && (!namespaceURI || (namespaceURI && namespaceURI == child.namespaceURI));
  },
  
  /**
   * Checks if the given node can contain the given XML.
   *
   * TODO: the parentNode should be needed here.
   */
  canInsertXML: function (theNode, theXML, parentNode)
  {
    if (theNode.nodeType != 1)
      return false;
    
    if (!parentNode)
      parentNode = theNode.parentNode;
    
    var nodeClone = theNode.cloneNode(true);
    var xmlClone  = theXML.cloneNode(true);
    
    parentNode.appendChild(nodeClone);
    var canInsertXML = this.xmlhandler.insertXML(nodeClone, xmlClone) && this.validator.isValid(nodeClone);
    parentNode.removeChild(nodeClone);
    
    return canInsertXML;
  },
  
  /**
   * Check if the given node can be inserted before the reference node in the given parent node.
   */
  canInsertAfter: function (theNode, parentNode, refNode)
  {
    if (parentNode.nodeType == 9)
      return false;
    
    var parentClone = parentNode.cloneNode(true);
    var refClone = null;
    if (refNode)
    {
      var refIdx = xopus_globs.utils.indexOf(refNode);
      
      refClone = parentClone.childNodes[refIdx];
    }
    
    var canInsertAfter = false;
    var parentIsDocumentElement = parentNode.parentNode.nodeType == 9;
    
    if (!parentIsDocumentElement)
      parentNode.parentNode.appendChild(parentClone);
      
    try
    {
      canInsertAfter = this.xmlhandler.insertNode(theNode, parentClone, refClone, true) && this.validator.isValid(parentClone);
    }
    finally
    {
      if (!parentIsDocumentElement)
        parentNode.parentNode.removeChild(parentClone);
    }
    
    return canInsertAfter;
  },
  
  /**
   * Checks if the given node can be moved up.
   */
  canMoveUp: function (theNode)
  {
    if (theNode.nodeType == 2 || !theNode.parentNode || theNode.parentNode.nodeType == 9)
      return false;
    
    // Check the cache.
    var xpath = this.xmlhandler.pipeline.getXPath(theNode);
    var canMoveUp = this.xmlhandler.xopus.cache.get(xpath, "4");
    
    if (canMoveUp == null)
    {
      var parentNode  = theNode.parentNode;
      var parentClone = parentNode.cloneNode(true);
      var childIdx    = xopus_globs.utils.indexOf(theNode);
      var childClone  = parentClone.childNodes[childIdx];

      var canMoveUp = false;
      var parentIsDocumentElement = parentNode.parentNode.nodeType == 9;

      if (!parentIsDocumentElement)
        parentNode.parentNode.appendChild(parentClone);

      try
      {
        canMoveUp = this.xmlhandler.moveUp(childClone, true) && this.validator.isValid(parentClone);
      }
      finally
      {
        if (!parentIsDocumentElement)
          parentNode.parentNode.removeChild(parentClone);
      }
      
      // Store in cache.
      this.xmlhandler.xopus.cache.put(xpath, "4", canMoveUp);
    }

    return canMoveUp;
  },

  /**
   * Checks if the given node can be moved down.
   */
  canMoveDown: function (theNode)
  {
    if (theNode.nodeType == 2 || !theNode.parentNode || theNode.parentNode.nodeType == 9)
      return false;
          
    // Check the cache.
    var xpath = this.xmlhandler.pipeline.getXPath(theNode);
    var canMoveDown = this.xmlhandler.xopus.cache.get(xpath, "5");
    
    if (canMoveDown == null)
    {
      var parentNode  = theNode.parentNode;
      var parentClone = parentNode.cloneNode(true);
      var childIdx    = xopus_globs.utils.indexOf(theNode);
      var childClone  = parentClone.childNodes[childIdx];

      var canMoveDown = false;
      var parentIsDocumentElement = parentNode.parentNode.nodeType == 9;

      if (!parentIsDocumentElement)
        parentNode.parentNode.appendChild(parentClone);

      try
      {
        canMoveDown = this.xmlhandler.moveDown(childClone, true) && this.validator.isValid(parentClone);
      }
      finally
      {
        if (!parentIsDocumentElement)
          parentNode.parentNode.removeChild(parentClone);
      }
      
      // Store in cache.
      this.xmlhandler.xopus.cache.put(xpath, "5", canMoveDown);
    }
    
    return canMoveDown;
  },

  /**
   * Checks if the given node can be deleted.
   */
  canDeleteNode: function (theNode)
  {
    if (theNode.nodeType != 2 && theNode.parentNode && theNode.parentNode.nodeType == 9)
      return false;
    
    // Check the cache.
    var xpath = this.xmlhandler.pipeline.getXPath(theNode);
    var canDeleteNode = this.xmlhandler.xopus.cache.get(xpath, "6");
    
    if (canDeleteNode == null)
    {
      var parentNode  = theNode.parentNode || theNode.selectSingleNode("..");
      var parentClone = parentNode.cloneNode(true);

      var childClone;
      if (theNode.nodeType == 2)
        childClone = parentClone.selectSingleNode("@*[local-name()='" + xopus_globs.utils.getLocalName(theNode) + "' and namespace-uri()='" + xopus_globs.utils.getNamespaceURI(theNode) + "']");
      else
        childClone = parentClone.childNodes[xopus_globs.utils.indexOf(theNode)];

      var canDeleteNode = false;
      var parentIsDocumentElement = parentNode.parentNode.nodeType == 9;

      if (!parentIsDocumentElement)
        parentNode.parentNode.appendChild(parentClone);

      try
      {
        canDeleteNode = this.xmlhandler.deleteNode(childClone, parentClone, true) && this.validator.isValid(parentClone);
      }
      finally
      {
        if (!parentIsDocumentElement)
          parentNode.parentNode.removeChild(parentClone);
      }
    
      // Store in cache.
      this.xmlhandler.xopus.cache.put(xpath, "6", canDeleteNode);
    }
    
    return canDeleteNode;
  },
  
  /**
   * Get the schema declaration for the given node.
   */
  getDeclaration: function (theNode)
  {
    return this.getSchemaCache().getDeclaration(theNode);
  },
  
  /**
   * Retrieve the possible children.
   *
   * TODO: support choices here
   * TODO: multiple equal localName with different namespace uris are not supported.
   */
  getPossibleChildren: function (theNode)
  {
    var possibleChildren = {};
    var decl = this.getSchemaCache().getDeclaration(theNode);

    if (!decl)
      __xopus_alert("!decl for: " + theNode.xml);

    // Only complex types can have child elements.
    if (decl.itemType == SOMITEM_ELEMENT && decl.type.itemType == SOMITEM_COMPLEXTYPE)
    {
      var theType = decl.type;

      if (theType.contentType == SCHEMACONTENTTYPE_ELEMENTONLY || theType.contentType == SCHEMACONTENTTYPE_MIXED)
        this.getPossibleChildrenForGroup(theType.contentModel, theNode, possibleChildren, 1);

      // Find the attributes for this element.
      this.getPossibleAttributes(theType, theNode, possibleChildren);
    }

    return possibleChildren;
  },

  /**
   * Get the possible children for the given group.
   */
  getPossibleChildrenForGroup: function (group, theNode, possibleChildren, maxOccurs)
  {
    maxOccurs = this.multiplyMaxOccurs(maxOccurs, group.maxOccurs);
    
    for (var i = 0; i < group.particles.length; i++)
    {
      var particle = group.particles.item(i);
      var maxParticleOccurs = this.multiplyMaxOccurs(maxOccurs, particle.maxOccurs);
      
      if ((particle.itemType & SOMITEM_GROUP) == SOMITEM_GROUP)
        this.getPossibleChildrenForGroup(particle, theNode, possibleChildren, maxOccurs);
      else if (theNode.selectNodes("*[local-name()='" + particle.name + "' and " +
                                     "namespace-uri()='" + particle.namespaceURI + "']").length < maxParticleOccurs)
        possibleChildren[particle.name] = particle;
    }
  },
  
  multiplyMaxOccurs: function (a, b)
  {
    return (a == -1 || b == -1) ? Infinity : a * b;
  },
  
  /**
   * Get the attributes for the given type. Will NOT find virtually defined (default) attributes.
   */
  getPossibleAttributes: function(theType, theNode, possibleChildren)
  {
    var attributes = theType.attributes;
    
    for (var i = 0; i < attributes.length; i++)
    {
      var attrDecl = attributes.item(i);
      var attr = theNode.selectSingleNode("@*[local-name()='" + attrDecl.name + "' and namespace-uri()='" + attrDecl.namespaceURI + "']");
      
      if (attr == null && attrDecl.use != SCHEMAUSE_PROHIBITED)
        possibleChildren[attrDecl.name] = attrDecl;
    }
  },
  
  /**
   * Get the declaration for the given child.
   */
  getChildDeclaration: function (theNode, localname, namespaceURI)
  {
    if (!namespaceURI)
      namespaceURI = "";
    
    var decl = this.getSchemaCache().getDeclaration(theNode);
    
    // Only complex types can have child elements.
    var result;
    if (decl.itemType == SOMITEM_ELEMENT && decl.type.itemType == SOMITEM_COMPLEXTYPE)
    {
      var theType = decl.type;
      
      if (theType.contentType == SCHEMACONTENTTYPE_ELEMENTONLY || theType.contentType == SCHEMACONTENTTYPE_MIXED)
        result = this.getChildDeclarationForGroup(theType.contentModel, theNode, localname, namespaceURI);
      
      if (!result)
        result = this.getAttributeDeclaration(theType, theNode, localname, namespaceURI);
    }
    
    return result;
  },

  /**
   * Search the possible child in the given group.
   */
  getChildDeclarationForGroup: function (group, theNode, localname, namespaceURI)
  {
    for (var i = 0; i < group.particles.length; i++)
    {
      var particle = group.particles.item(i);

      if ((particle.itemType & SOMITEM_GROUP) == SOMITEM_GROUP)
      {
        var decl = this.getChildDeclarationForGroup(particle, theNode, localname, namespaceURI);
        if (decl)
          return decl;
      }
      else if (particle.name == localname && (particle.namespaceURI == null ? "" : particle.namespaceURI) == namespaceURI && 
              (particle.maxOccurs == -1 || theNode.selectNodes(particle.name).length < particle.maxOccurs))
        return particle;
    }
    
    return null;
  },
  
  /**
   * Search for an attribute with the given name and namespace. Will also find attributes already defined virtually.
   */
  getAttributeDeclaration: function (theType, theNode, localname, namespaceURI)
  {
    var attributes = theType.attributes;
    
    for (var i = 0; i < attributes.length; i++)
    {
      var attrDecl = attributes(i);
      var attr = theNode.selectSingleNode("@*[local-name()='" + attrDecl.name + "' and namespace-uri()='" + attrDecl.namespaceURI + "']");
      
      if ((!attr || !attr.specified) && attrDecl.use != SCHEMAUSE_PROHIBITED && attrDecl.name == localname && attrDecl.namespaceURI == namespaceURI)
        return attrDecl;
    }
    
    return null;
  },
  
  /**
   * Create a node for the given schema declaration.
   */
  createNodeFromDeclaration: function(decl, parentNode)
  {
    switch (decl.itemType)
    {
      case SOMITEM_ELEMENT:
        return this.createElementFromDeclaration(decl, parentNode);
      case SOMITEM_ATTRIBUTE:
        return this.createAttributeFromDeclaration(decl, parentNode);
      default:
        __xopus_alert("Unknown declaration type: " + decl.itemType);
        return null;
    }
  },
  
  /**
   * Helper function.
   */
  createNode: function(type, decl, parentNode)
  {
    var namespaceURI = xopus_globs.utils.getNamespaceURI(decl);
    var prefix = xopus_globs.utils.getPrefixForNamespace(parentNode, namespaceURI);
    
    var name = decl.name;
    if (prefix)
      name = prefix + ":" + name;
    
    return this.theDoc.createNode(type, name, namespaceURI);
  },
  
  /**
   * Create an element for the given schema declaration.
   */
  createElementFromDeclaration: function(decl, parentNode)
  {
    // Create a new element in the specified namespace.
    var newChild = this.createNode(1, decl, parentNode);
    
    var theType = decl.type;
    
    // Fire events
    var res = this.fireEvent("onCreate", decl, newChild);
    if (res == SE_STOP)
      return newChild;
    else if (res == SE_CANCEL)
      return null;
      
    res = this.fireEvent("onCreate", decl.type, newChild);
    if (res == SE_STOP)
      return newChild;
    else if (res == SE_CANCEL)
      return null;
    
    // Only complex types can have child elements.
    if (theType.itemType == SOMITEM_COMPLEXTYPE)
    {
      if (theType.contentType == SCHEMACONTENTTYPE_ELEMENTONLY || theType.contentType == SCHEMACONTENTTYPE_MIXED)
        if (!this.createNodeFromGroupDeclaration(theType.contentModel, newChild, parentNode))
          return null;
      
      // Create all required attributes.
      if (!this.createRequiredAttributes(theType, newChild, parentNode))
        return null;
    }
    
    return newChild;
  },
  
  /**
   * Create all attributes an element should have according to the schema.
   */
  createRequiredAttributes: function (theType, theNode, parentNode)
  {
    var attributes = theType.attributes;
        
    for (var i = 0; i < attributes.length; i++)
    {
      var attrDecl = attributes.item(i);
      
      if (attrDecl.use == SCHEMAUSE_REQUIRED)
      {
        var attr = this.createAttributeFromDeclaration(attrDecl, parentNode);
        
        // The creation might have been cancelled
        if (!attr)
          return false;
        
        theNode.setAttributeNode(attr);
      }
    }
    
    return true;
  },
  
  /**
   * Create an attribute for the given schema declaration.
   */
  createAttributeFromDeclaration: function(decl, parentNode)
  {
    // Create a new attribute in the specified namespace.
    var newChild = this.createNode(2, decl, parentNode);
    
    // Fire event
    var res = this.fireEvent("onCreate", decl, newChild);
    if (res == SE_STOP)
      return newChild;
    else if (res == SE_CANCEL)
      return null;
    
    if (decl.fixedValue)
      newChild.nodeValue = decl.fixedValue;
    else if (decl.defaultValue)
      newChild.nodeValue = decl.defaultValue;
    else
      newChild.nodeValue = "new " + decl.name;
      
    return newChild;
  },
    
  /**
   * Create a node for the given schema declaration.
   */
  createNodeFromGroupDeclaration: function(group, newChild, parentNode)
  {
    if (group.minOccurs == 0)
      return true;
    
    for (var i = 0; i < group.particles.length; i++)
    {
      // TODO make this user selectable
      if (group.itemType == SOMITEM_CHOICE && i == 1)
        return true;
      
      var particle = group.particles.item(i);

      for (var j = 0; j < particle.minOccurs; j++)
      {
        if ((particle.itemType & SOMITEM_GROUP) == SOMITEM_GROUP)
          this.createNodeFromGroupDeclaration(particle, newChild, parentNode);
        else
        {
          var particleNode = this.createNodeFromDeclaration(particle, parentNode);
          
          // The creation might have been cancelled
          if (!particleNode)
            return false;
           
          newChild.appendChild(particleNode);
        }
      }
    }    
    
    return true;
  },
  
  /**
   * Fire schema event.
   *
   * name  The name of the event to fire
   * decl  The schema declaration where the event should be declared
   * node  The node to give as parameter to the event handler
   */
  fireEvent: function (name, decl, node)
  {
    // Find the annotation.
    var ann = decl.annotation;    
    if (!ann)
      return;
    
    // Find the event we're looking for.
    var event = ann.selectSingleNode(".//*[local-name()='events' and namespace-uri()='http://www.xopus.org/xmlns/xsd']"+
                                     "/*[local-name()='event' and namespace-uri()='http://www.xopus.org/xmlns/xsd']" +
                                     "[@name='" + name + "']");

    // @@ Sven
    if (!event) {
      // Register the last created node.
      // This node will be used as parent node for the next node
      window.lastCreatedNode_RUG = node;

      return;
    }

    var extension = event.getAttribute("extension");
    var handler   = event.getAttribute("handler");
    
    return xopus_globs.schemaExtensions.call(extension, handler, node);
  },
  
  /**
   * Loads all imports of the given XSD. xsdURI is the URI of the given xsd.
   */
  loadImports: function (theXsd, xsdURI)
  {
    // Find all imports
    var imports = theXsd.selectNodes("//*[local-name()='import'][namespace-uri()='http://www.w3.org/2001/XMLSchema']");
    
    for (var i = imports.length - 1; i >= 0; i--)
    {
      var importElt = imports[i];
      
      // Load the imported schema.
      var schemaLocation = importElt.getAttribute("schemaLocation");
      
      if (!schemaLocation)
        continue;
      
      var newURI = xopus_globs.utils.getAbsoluteURI(xsdURI, schemaLocation);
      var namespace = importElt.getAttribute("namespace");
      
      this.loadSchema(newURI, namespace);
    }
  },
  
  /**
   * Resolves all includes of the given XSD. xsdURI is the URI of the given xsd.
   */
  resolveIncludes: function (theXsd, xsdURI)
  {
    // Find all includes
    var includes = theXsd.selectNodes("//*[local-name()='include'][namespace-uri()='http://www.w3.org/2001/XMLSchema']");
    
    for (var i = includes.length - 1; i >= 0; i--)
    {
      var include = includes[i];
     
      // Load the included schema.
      var schemaLocation = include.getAttribute("schemaLocation");
      var newURI = xopus_globs.utils.getAbsoluteURI(xsdURI, schemaLocation);
      var newXsd = xopus_globs.datadriver.loadSchema(newURI);
      var parent = theXsd.documentElement;
      
      // Apply IE bug workaround
      this.fixGroups(newXsd);

      // Resolve all includes
      this.resolveIncludes(newXsd, newURI);    
      
      // Replace the include tag with all children of the schema.
      var children = newXsd.selectNodes("/*[local-name()='schema'][namespace-uri()='http://www.w3.org/2001/XMLSchema']/*[namespace-uri()='http://www.w3.org/2001/XMLSchema']");

      for (var j = 0; j < children.length; j++)
      {
        var child = children[j];
        var namespaceDeclarations = child.selectNodes("namespace::node()");

        // Add namespace declarations for all namespaces (except xml namespace) and prefixes known in the template.
        for (var k = 0; k < namespaceDeclarations.length; k++)
          if (namespaceDeclarations[k].nodeValue != "http://www.w3.org/XML/1998/namespace")
            child.setAttributeNode(namespaceDeclarations[k].cloneNode(true));
        
        // Check if the child is an import.
        if (xopus_globs.utils.getLocalName(child) == "import")
        {
          child.setAttribute("schemaLocation", xopus_globs.utils.getAbsoluteURI(newURI, child.getAttribute("schemaLocation")));
          parent.insertBefore(child, parent.firstChild);
        }
        else
        {
          parent.appendChild(children[j].cloneNode(true));
        }
      }
      
      // Remove the include to prevent the SOM from loading it.
      parent.removeChild(include);
    }
  },
  
  /**
   * This is a workaround for a bug in MSXML 4.0 and MSXML 4.0 SP1.
   */
  fixGroups: function (doc)
  {
    if (xopus_globs.useXopusSOM)
      return;
    
    var groupChildren = doc.selectNodes("//*[local-name()='group'][namespace-uri()='http://www.w3.org/2001/XMLSchema']/*[@minOccurs != '' or @maxOccurs != '']");
    
    for (var i = 0; i < groupChildren.length; i++)
    {
      var groupChild = groupChildren[i];
      
      // Create a new sequence element.
      var sequence = doc.createNode(1, "sequence", "http://www.w3.org/2001/XMLSchema");
      sequence.appendChild(groupChild.cloneNode(true));
      
      // Replace the choice with the seqence
      groupChild.parentNode.replaceChild(sequence, groupChild);
    }
  }
};

numScriptsLoading--;