////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

function Toolbar(logoMenuDefinition, toolbarDefinition)
{
  //some browser specific dimensions
  if (xopus_globs.browser.ie) {
    this.offsets = [-3, -3, -4, -5];
    this.dims = [32, 32, 33, 34];
  } else {
    this.offsets = [0, 0, 0, 0];
    this.dims = [30, 32, 29, 32];
  }

  //constants
  this.MENUALIGNMENTPOINTS = [xopus_consts.BOTTOMLEFT, xopus_consts.LEFTTOP, xopus_consts.TOPLEFT, xopus_consts.TOPRIGHT];

  //properties
  this.logoMenuDefinition = logoMenuDefinition;
  this.logoMenuDefinition.showAsMenu = true;
  this.toolbarDefinition = toolbarDefinition;
  this.mainBodyEl = xopus_globs.window.document.getElementById('site');

  //create a toolbar container
  var div = xopus_globs.window.document.createElement('div');
  div.id = "xopus_toolbar";
  div.className = "xopus_toolbar_none";
  div.unselectable = "on";
  this.toolbarEl = xopus_globs.window.document.body.appendChild(div);

  //create toolbar content
  var s = ''
    + '<img id="xopus_toolbar_bgstart" src="' + xopus_globs.basePath + 'xopus/media/nix.gif" unselectable="on" />'
    + '<img id="xopus_toolbar_bgend" src="' + xopus_globs.basePath + 'xopus/media/nix.gif" unselectable="on" />'
    + '<img id="xopus_toolbar_xopuslogo" oriclass="xopus_toolbar_xopuslogo" class="xopus_toolbar_xopuslogo" src="' + xopus_globs.basePath + 'xopus/media/nix.gif" unselectable="on" />'
    + '<nobr>'
    + '  <div id="xopus_toolbar_bg" unselectable="on"></div>'
    + '  <div id="xopus_toolbar_groups" unselectable="on"></div>'
    //add a nodeName-indicator
    + '<div id="xopus_toolbar_status" unselectable="on"></div>'
    + '</nobr>';
  this.toolbarEl.innerHTML = s;

  //properties
  this.busy = 0;
  this.neverShown = true;
  this.hidden = true;
  this.bgEl = xopus_globs.window.document.getElementById('xopus_toolbar_bg');
  this.logoEl = xopus_globs.window.document.getElementById('xopus_toolbar_xopuslogo');
  this.groupsEl = xopus_globs.window.document.getElementById('xopus_toolbar_groups');
  this.statusEl = xopus_globs.window.document.getElementById('xopus_toolbar_status');
  this.statusValue = null;
  this.defaultStatusValue = null;
  this.statusTOH = null;
  this.statusVisible = (('' + xopus_globs['TB.statusVisible']) == 'true');
  this.statusEl.style.display = this.statusVisible?'block':'none';

  //determine the position to use
  this.prevPosi = -1;
  this.curPosi = xopus_globs['TB.curPosi'];
  if (isNaN(this.curPosi)) this.curPosi = 2;

  //create all groups and buttons
  this.createGroups();

  //attach some event handlers
//TODO!!!!! Get rid of attachEvents!
  xopus_globs.framework.attachEvent(this.toolbarEl, "oncontextmenu", this, "doContextMenuToolbar");
  xopus_globs.framework.attachEvent(this.toolbarEl, (xopus_globs.browser.ie?'ondragstart':'ondraggesture'), xopus_globs.framework, 'cancelEvent');
  var logo = xopus_globs.window.document.getElementById('xopus_toolbar_xopuslogo');
  xopus_globs.framework.attachEvent(logo, "oncontextmenu", this, "doContextMenuToolbar");
  xopus_globs.framework.attachEvent(logo, "onclick", xopus_globs.toolbardefinition, "doClickLogo");

  //create toolbar context-menu definition
  //the toolbar's contextmenu will consist of:
  //  the 4 possible directions for the toolbar: top left bottom right
  //  a separator
  //  the groups that have been defined in toolbarDefinition
  var menuTxt = xopus_globs.locale.getString('Menu');
  this.cmItems = {id:'toolbar'};
  this.cmItems.items = {};
  for (var i=0; i<4; i++) {
    var pos = xopus_consts.i2pos[i];
    this.cmItems.items['TB' + pos] = { icon:xopus_globs.basePath + 'xopus/toolbar/media/icons/toolbar' + pos + '.gif', disabled:false, bullited:(i==this.curPosi), obj:this, onclick:'doPosition' };
  }
  this.cmItems.items['separator1'] = {};
  this.cmItems.items['status'] = { checked:this.statusVisible, obj:this, onclick:'toggleStatusVisibility' };
  this.cmItems.items['separator2'] = {};
  for (var id in this.toolbarDefinition) {
    var group = this.toolbarDefinition[id];
    this.cmItems.items['TG' + id] = { checked:true, disabled:group.neverHide, obj:this, onclick:'toggleGroup' };
  }

  //subscribe to enabled settings and transform shortcuts
  for (var groupid in this.toolbarDefinition) {
    var group = this.toolbarDefinition[groupid];
    if (group.items) {
      for (var itemid in group.items) {
        var item = group.items[itemid];
        item.id = itemid;
        item.shortcutArray = xopus_globs.utils.shortcutsToArray(item.shortcuts);
        xopus_globs.subscribe('toolbarDefinition.' + groupid + '.items.' + itemid + '.enabled', this, 'handleEnabledChange');
      }
    }
  }

  //suscribe to a language change
  xopus_globs.subscribe('language', this, 'applyLanguage');
  this.applyLanguage();

  //suscribe to a xopusStarted change
  xopus_globs.subscribe('xopusStarted', this, 'applyXopusStarted');
}

Toolbar.prototype=
{
  doResize: function () {
    var pos = xopus_consts.i2pos[this.curPosi];
    this.mainBodyEl.style.width = '100%';
    this.mainBodyEl.style.height = '100%';

    if (!this.hidden) {
      this.mainBodyEl.style[this.curPosi] = (this.offsets[this.curPosi] + xopus_consts.i2dim[this.curPosi]) + 'px';
      this.mainBodyEl.style[xopus_consts.i2dim[this.curPosi]] = Math.max(0, (xopus_globs.window.document.body['offset' + xopus_consts.i2Dim[this.curPosi]] - (this.dims[this.curPosi] + this.offsets[this.curPosi]))) + 'px';
    }
  },
  doPosition: function(pos) {
    if (pos && (pos.indexOf('TB') == 0)) pos = pos.substring(2);
    if ((this.busy > 0) || (xopus_consts.pos2i[pos] == this.curPosi)) return;
    this.busy++;

    //hide the contextmenu
    xopus_globs.contextmenu.hide();

    //get position to display toolbar
    if (typeof(pos) == 'undefined') var pos = xopus_consts.i2pos[this.curPosi];
    else this.curPosi = xopus_consts.pos2i[pos];

    //store the new value as the new default
    xopus_globs.store('TB.curPosi', this.curPosi);

    //reset alignment point for logomenu
    this.logoMenuDefinition.alignment = this.MENUALIGNMENTPOINTS[this.curPosi];

    if (this.prevPosi >= 0) {
      this.hide(true);
    } else {
      this.mainBodyEl.className = 'xopus_mainbody_' + pos;
      this.toolbarEl.className = 'xopus_toolbar_' + pos;
      if (!this.hidden) this.show();
    }

    //store previous position
    this.prevPosi = this.curPosi;
  },
  show:function() {
    this.hidden = false;
    if (this.neverShown) {
      this.neverShown = false;
      xopus_globs.toolbar.doPosition();
      if (xopus_globs.browser.moz) xopus_globs.toolbar.createGroups();
      return;
    }

    //find words for the position
    var pos = xopus_consts.i2pos[this.curPosi];
    var dim = xopus_consts.i2dim[this.curPosi];
    var Dim = xopus_consts.i2Dim[this.curPosi];

    var delay = 500;
    var duration = 900;

    var startCode = ((this.prevPosi < 0)?'':''
      + 'xopus_globs.toolbar.createGroups();'
      + 'var tb = xopus_globs.toolbar.toolbarEl;'
      + 'tb.className = "xopus_toolbar_' + pos + '";'
      + 'tb.style.top="auto"; tb.style.left="auto"; tb.style.bottom="auto"; tb.style.right="auto"; tb.style.' + pos + '="-' + this.dims[this.curPosi] + 'px";');
//__xopus_alert('startCode = ' + startCode);
//if (this.prevPosi < 0) {
  var tb = this.toolbarEl;
  tb.className = 'xopus_toolbar_' + pos;
  tb.style.top='auto'; tb.style.left='auto'; tb.style.bottom='auto'; tb.style.right='auto';
  tb.style[pos] = (-this.dims[this.curPosi]) + 'px';
//}
//__xopus_alert(this.curPosi + ', ' + pos);
//return;
    var endCode = ''
      + 'var mb = xopus_globs.toolbar.mainBodyEl;'
      + 'mb.style.'+pos+'="'+(this.dims[this.curPosi] + this.offsets[this.curPosi])+'px";'
      + 'mb.style.'+dim+'="'+(xopus_globs.window.document.body['offset' + Dim] - (this.dims[this.curPosi] + this.offsets[this.curPosi]))+'px";'
      + 'xopus_globs.toolbar.busy--;'
      + 'xopus_globs.eventCenter.doResizeXopusWindow();';
    xopus_globs.animator.start([{ els:this.toolbarEl, anims:{ prop:[pos], target:[this.offsets[this.curPosi]], startCode:[startCode], endCode:[endCode], delay:[delay], duration:[duration], profile:[xopus_globs.animator.BOUNCE] }}], 'xopus_toolbar');
  },
  hide:function(showAgain) {
    this.hidden = true;

    //find words for the position
    var prevpos = xopus_consts.i2pos[this.prevPosi];
    var prevdim = xopus_consts.i2dim[this.prevPosi];
    var prevDim = xopus_consts.i2Dim[this.prevPosi];

    var delay = 200;
    var duration = 900;
    xopus_globs.toolbar.mainBodyEl.style[prevpos] = '0px';
    xopus_globs.toolbar.mainBodyEl.style[prevdim] = (xopus_globs.window.document.body['offset' + prevDim]) + 'px';
    xopus_globs.animator.start([{ els:this.toolbarEl, anims:{ prop:[prevpos], endCode:(showAgain?'xopus_globs.toolbar.show();':''), target:[-this.dims[this.curPosi]], delay:[delay], duration:[duration], profile:[xopus_globs.animator.BOUNCE]}}], 'xopus_toolbar');
  },
  doMouseOverItem:function(el) {
    if (el.className == 'xopus_toolbar_item') {
      el.className = 'xopus_toolbar_item_over';

      //call the onmouseover handler, if present
      var groupID = el.getAttribute('groupID');
      var itemID = el.getAttribute('itemID');
      var item = this.toolbarDefinition[groupID].items[itemID];

      //execute onmouseover handler
      xopus_globs.utils.runHandler(item, 'onmouseover');
    }
  },
  doMouseOutItem:function(el) {
    if (el.className == 'xopus_toolbar_item_over') {
      el.className = 'xopus_toolbar_item';

      //call the onmouseover handler, if present
      var groupID = el.getAttribute('groupID');
      var itemID = el.getAttribute('itemID');
      var item = this.toolbarDefinition[groupID].items[itemID];

      //execute onmouseover handler
      xopus_globs.utils.runHandler(item, 'onmouseout');
    }
  },
  doClickItem:function(el, evt) {
    if (el.className == 'xopus_toolbar_item_dis') return;

    el.className = 'xopus_toolbar_item_down';

    var groupID = el.getAttribute('groupID');
    var itemID = el.getAttribute('itemID');
    var item = this.toolbarDefinition[groupID].items[itemID];
    item.id = itemID;
    item.el = el;

    //execute onclick handler
    var theEvent = {type:evt.type, srcElement:evt.srcElement, target:evt.target, clientX:evt.clientX, clientY:evt.clientY};
    setTimeout(
      function() {
        xopus_globs.utils.runHandler(item, 'onclick', item.id, theEvent);
        if (!item.hasMenu) setTimeout(function(){if (el.className == 'xopus_toolbar_item_down') el.className = 'xopus_toolbar_item';}, 0);
      }, 0);
  
    xopus_globs.framework.cancelEvent(evt);
  },
  doContextMenuToolbar:function(evt, dontMove) {
//TODO!!!
    //don't do a thing if a dialog is visible
    if (xopus_globs.dialog.isVisible) return false;

    var el = evt.srcElement;

    //cancel the standard handling of this event
    xopus_globs.framework.cancelEvent(evt);

    //set the contextmenu items correct
    var curPos = xopus_consts.i2pos[this.curPosi];
    for (var i=0; i<4; i++) {
      var pos = xopus_consts.i2pos[i];
      this.cmItems.items['TB' + pos].bullited = (pos == curPos);
    }
    this.cmItems.items['status'].checked = this.statusVisible;
    for (var id in this.toolbarDefinition) {
      var group = this.toolbarDefinition[id];
      this.cmItems.items['TG' + id].checked = xopus_globs['TB.' + id];
      this.cmItems.items['TG' + id].disabled = (group.neverHide && xopus_globs['TB.' + id]);
    }
    xopus_globs.contextmenu.draw(xopus_globs.window, evt, this.cmItems, true, dontMove);
  },
  doContextMenuGroup:function(evt, dontMove) {
//TODO!!!
    //don't do a thing if a dialog is visible
    if (xopus_globs.dialog.isVisible) return false;

    var el = evt.srcElement;
    if (!el || (el.id.indexOf('xopus_toolbar_') != 0)) return;

    //cancel the standard handling of this event
    xopus_globs.framework.cancelEvent(evt);

    var groupID = el.getAttribute('groupID');

    //create a context menu
    var group = this.toolbarDefinition[groupID];
    var contextmenuDefinition = {id:groupID};
    contextmenuDefinition.items = {};

    contextmenuDefinition.items['TG' + groupID] = { name:(xopus_globs.locale.getString('TG' + groupID)), disabled:(group.neverHide || !xopus_globs['TB.' + groupID]), checked:true, obj:this, onclick:'toggleGroupAndHide' };
    contextmenuDefinition.items['separator'] = { };
    for (var itemID in group.items) {
      var item = group.items[itemID];
      if (item.neverShow) continue;
      
      var checked = xopus_globs['TB.' + itemID] && !item.neverShow;
      contextmenuDefinition.items[itemID] = {
        disabled:((item.neverShow == true) && !checked),
        checked:checked,
        icon: xopus_globs.basePath + 'xopus/media/icons/' + itemID + '.gif',
        obj:this,
        onclick:'doContextmenuItemClick'
      };
    }
    xopus_globs.contextmenu.draw(xopus_globs.window, evt, contextmenuDefinition, true, dontMove);
  },
  doContextmenuItemClick:function(id) {
//TODO!!!
    //don't do a thing if a dialog is visible
    if (xopus_globs.dialog.isVisible) return false;

    var makeVisible = !xopus_globs['TB.' + id];

    //find item element
    var itemEl = xopus_globs.window.document.getElementById('xopus_toolbar_item_' + id);

    //find the group
    var groupID = itemEl.getAttribute('groupID');
    var group = this.toolbarDefinition[groupID];

    //see if this item needs to be hidden and if so whether it is the last visible item left
    //if so hide the group instead of the item
    var lastOne = false;
    if (!makeVisible) {
      lastOne = true;
      for (var itemID in group.items) {
        var item = group.items[itemID];
        if ((itemID != id) && (!item.hidden)) {
          lastOne = false;
          break;
        }
      }
    }

    if (lastOne) {
      //hide if allowed
      if (!group.neverHide) this.toggleGroup(groupID, true);
    } else {
      this.busy++;

      //store the new value
      xopus_globs.store('TB.' + id, makeVisible);

      //find the item
      var itemID = itemEl.getAttribute('itemID');
      var item = this.toolbarDefinition[groupID].items[itemID];

      item.hidden = !makeVisible;

      //determine state of this item
      var state = '';
      if (!makeVisible) state = '_hidden';
      else if (!item.obj || !item.onclick) state = '_dis';

      //set the new state of the item by setting the classname
      var target = makeVisible?20:0;
      var profile = (makeVisible?xopus_globs.animator.NORMAL:xopus_globs.animator.DECELERATE);
      itemEl.style.visibility = 'visible';
      if (makeVisible) {
        itemEl.style.width = '0px';
        itemEl.style.height = '0px';
      }
      xopus_globs.animator.start([{ els:itemEl, anims:{ prop:['width', 'height'], target:[target], endCode:'el.style.visibility="visible"; el.className="xopus_toolbar_item' + state + '"; xopus_globs.toolbar.busy--;', delay:500, duration:[400], profile:[profile] }}], 'xopus_toolbar');

      //open the contextmenu again
      this.doContextMenuGroup({srcElement:group.el}, true);
    }
  },
  toggleGroup:function(id, dontOpenContextmenu) {
    this.busy++;

    var pos = xopus_consts.i2pos[this.curPosi];
    var perpdim = xopus_consts.i2perpdim[this.curPosi];

    if (id && (id.indexOf('TG') == 0)) id = id.substring(2);
    var makeVisible = !xopus_globs['TB.' + id];
//__xopus_alert((makeVisible?'showing ':'hiding ') + id + ' group');
    xopus_globs.store('TB.' + id, makeVisible);
    var groupEl = xopus_globs.window.document.getElementById('xopus_toolbar_group_' + id);

    var newClassName = 'xopus_toolbar_group' + (makeVisible?'':'_dis');

    //find the group definition
    var groupID = groupEl.getAttribute('groupid');
    var group = this.toolbarDefinition[groupID];

    //can't get Moz to animate, yet
    if (xopus_globs.browser.moz) {
      groupEl.className = newClassName;
      this.busy--;
    } else {
      //animations in IE are dangerous! you shouldn't change toolbar position afterwards or you might loose all groups
      groupEl.style.overflow = 'hidden';
      groupEl.style.display = 'inline';

      var target = (makeVisible?this.calculateGroupSize(group):1);
      var profile = (makeVisible?xopus_globs.animator.NORMAL:xopus_globs.animator.DECELERATE);
      xopus_globs.animator.start([{ els:groupEl, anims:{ prop:[perpdim], target:[target], endCode:'xopus_globs.toolbar.busy--;', duration:[800], profile:[profile] }}], 'xopus_toolbar');
    }

    //open contextmenu again
    if (!dontOpenContextmenu) this.doContextMenuToolbar({srcElement:this.toolbarEl}, true)
  },
  toggleGroupAndHide:function(id) {
    this.toggleGroup(id, true);
    xopus_globs.contextmenu.hide();
  },
  calculateGroupSize:function(group) {
    //calculate the width of the group
    var size = 4;
    for (var itemid in group.items) if (!group.items[itemid].hidden) size += 22;
    return size;
  },
  createGroups:function() {
    var theDir = xopus_consts.i2dir[this.curPosi];

    //change logo
    this.logoEl.src = xopus_globs.basePath + 'xopus/toolbar/media/toolbar_xopuslogo_' + theDir + '.gif';

    //create bg image
    if (xopus_globs.browser.ie) {
      var s = '';
      if (theDir == 'ver') {
        s = '<img style="position:absolute; top:' + ((this.curPosi==0)?1:0) + 'px; left:0px; z-index:2;" src="' + xopus_globs.basePath + 'xopus/toolbar/media/toolbar_bgstart_hor_ie.gif" width="7" height="30" />'
        + '<img style="position:absolute; top:' + ((this.curPosi==0)?3:2) + 'px; left:0px; z-index:1;" src="' + xopus_globs.basePath + 'xopus/toolbar/media/toolbar_bg_hor_ie.gif" width="100%" height="26" />'
        + '<img style="position:absolute; top:' + ((this.curPosi==0)?1:0) + 'px; right:-1px; z-index:2;" src="' + xopus_globs.basePath + 'xopus/toolbar/media/toolbar_bgend_hor_ie.gif" width="2" height="30" />';
      } else {
        s = '<img style="position:absolute; top:0px; left:' + ((this.curPosi==1)?-2:0) + 'px; z-index:2;" src="' + xopus_globs.basePath + 'xopus/toolbar/media/toolbar_bgstart_ver_ie.gif" width="36" height="7" />'
        + '<img style="position:absolute; top:0px; left:' + ((this.curPosi==1)?3:5) + 'px; z-index:1;" src="' + xopus_globs.basePath + 'xopus/toolbar/media/toolbar_bg_ver_ie.gif" width="26" height="100%" />'
        + '<img style="position:absolute; bottom:-1px; left:' + ((this.curPosi==1)?-2:0) + 'px; z-index:2;" src="' + xopus_globs.basePath + 'xopus/toolbar/media/toolbar_bgend_ver_ie.gif" width="36" height="2" />';
      }
      this.bgEl.innerHTML = s;
    }

    //loop groups
    var s = '';
    for (var groupid in this.toolbarDefinition) {
      var group = this.toolbarDefinition[groupid];
      s += '<div style="font-size:1px;" id="xopus_toolbar_group_' + groupid + '" groupID="' + groupid + '" class="xopus_toolbar_group' + (xopus_globs['TB.' + groupid]?'':'_dis') + '" unselectable="on">';
      for (var itemid in group.items) {
        var item = group.items[itemid];
        if (item.neverShow) continue;

        var state = this.getState(itemid, item);

        var imgSrc = xopus_globs.useMozOpacity?itemid:(state != '_dis'?'nix':'raster2');
        var imgStyle = xopus_globs.useMozOpacity?'':(' style="background-image:url(' + xopus_globs.basePath + 'xopus/media/icons/' + itemid + '.gif)" ');
        s += '<img src="' + xopus_globs.basePath + 'xopus/media/icons/' + imgSrc + '.gif" ' + imgStyle + ' groupID="' + groupid + '" itemID="' + itemid + '" id="xopus_toolbar_item_' + itemid + '" oriclass="xopus_toolbar_item" class="xopus_toolbar_item' + state + '" titleid="' + itemid + '" unselectable="on" />';
      }

      s += '<div id="xopus_toolbar_separator_' + groupid + '" class="xopus_toolbar_separator" unselectable="on"><img src="' + xopus_globs.basePath + 'xopus/toolbar/media/separator_' + theDir + '.gif" title="" unselectable="on" /></div>';
      s += '</div>';
    }
    this.groupsEl.innerHTML = s;

    //place all texts in the correct language
    this.applyLanguage();

    //attach events to these groups
    for (var groupid in this.toolbarDefinition) {
      var group = this.toolbarDefinition[groupid];
      group.el = xopus_globs.window.document.getElementById('xopus_toolbar_group_' + groupid);
      xopus_globs.framework.attachEvent(group.el, "oncontextmenu", this, "doContextMenuGroup");
    }
  },
  getState:function(itemid, item) {
    //determine state of this item
    var state = '';
    if (!xopus_globs['TB.' + itemid]) state = '_hidden';
    else if (item.enabled != true) state = '_dis';
    else if (item.down) state = '_down';
    return state;
  },
  applyXopusStarted:function() {
    if (xopus_globs.xopusStarted) this.show();
    else this.hide();
  },
  applyLanguage:function() {
    //get the locale object
    var locale = xopus_globs.locale;

    var menuDefs = [this.logoMenuDefinition, this.cmItems];

    //set all name values in all menu definitions
    for (var i=0; i<menuDefs.length; i++) {
      if (!menuDefs[i]) continue;
      var items = menuDefs[i].items;
      if (!items) continue;
      for (var itemid in items) {
        var item = items[itemid];
        item.name = xopus_globs.utils.capitalize(locale.getString(itemid));

        if (item.menu) {
          for (var subitemid in item.menu.items) {
            var subitem = item.menu.items[subitemid];
            subitem.name = locale.getString(subitemid);
          }
        }
      }
    }

    //set all individual elements title property

    //scan all images inside the toolbar for a titleid property
    var imgs = this.toolbarEl.getElementsByTagName('img');
    for (var i=0; i<imgs.length; i++) {
      var img = imgs[i];
      var titleid = img.getAttribute('titleid');
      if (titleid) {
        var itemid = img.getAttribute('itemID');
        if (itemid) {
          var groupid = img.getAttribute('groupID');
          var item = this.toolbarDefinition[groupid].items[itemid];
          if (item) img.setAttribute('title', locale.getString(titleid) + xopus_globs.utils.shortcutsToString(item.shortcuts));
        } else {
          img.setAttribute('title', locale.getString(titleid));
        }
      }
    }
  },
  setEnabled:function(groupid, itemid, enabled) {
    var itemEl = xopus_globs.window.document.getElementById('xopus_toolbar_item_' + itemid);
    if (!itemEl) return;
    var item = this.toolbarDefinition[groupid].items[itemid];
    if (item) {
      this.toolbarDefinition[groupid].items[itemid].enabled = (enabled && true && ((typeof(item.href) != "undefined") || (typeof(item.onclick) != "undefined")));

      var state = this.getState(itemid, item);
      if ((item.hidden != true) && (itemEl.className != 'xopus_toolbar_item' + state)) {
        itemEl.className = 'xopus_toolbar_item' + state;
        
        if (!xopus_globs.useMozOpacity) {
          if (state == '_dis') {
            itemEl.src = xopus_globs.basePath + 'xopus/media/icons/raster2.gif';
          } else {
            itemEl.src = xopus_globs.basePath + 'xopus/media/icons/nix.gif';
          }
        }
      }
    }
  },
  handleEnabledChange:function(thePath, theValue) {
    var parts = thePath.split('.');
    var itemid = parts[parts.length-2];
    var itemEl = xopus_globs.window.document.getElementById('xopus_toolbar_item_' + itemid);
    if (!itemEl) return;
    var groupid = itemEl.getAttribute('groupID');

    this.setEnabled(groupid, itemid, theValue);
  },
  setStatusValue:function(theValue) {
    if (theValue != this.statusValue) {
      if ((theValue && true) != (this.statusValue && true)) this.statusEl.style.visibility = (theValue?'visible':'hidden');
      if (theValue) this.statusEl.innerHTML = '<span>' + theValue + '</span>';
      this.statusValue = theValue;
    }

    if (this.statusTOH) clearTimeout(this.statusTOH);
    this.statusTOH = null;
    if (theValue != this.defaultStatusValue) this.statusTOH = setTimeout('xopus_globs.toolbar.resetStatusValue();', 3000);
  },
  setStatusDefaultValue:function(theValue) {
    this.defaultStatusValue = theValue;
    this.setStatusValue(theValue);
  },
  resetStatusValue:function() {
    if (this.statusTOH) clearTimeout(this.statusTOH);
    this.statusTOH = null;
    this.setStatusValue(this.defaultStatusValue);
  },
  toggleStatusVisibility:function() {
    this.statusVisible = !this.statusVisible;
    xopus_globs.store('TB.statusVisible', this.statusVisible);
    this.statusEl.style.display = this.statusVisible?'block':'none';

    //open contextmenu again
    this.doContextMenuToolbar({srcElement:this.toolbarEl}, true);
  },

  doKeyDown:function(evt) {
    //see if the control key or a special key was pressed
//if ((evt.ctrlKey || evt.metaKey) && ((evt.keyCode > 32) || (evt.charCode > 32))) __xopus_alert(evt.keyCode + ', ' + evt.charCode);
    if ((evt.ctrlKey || evt.metaKey) && (evt.keyCode >= 32)) {
      //loop groups
      for (var groupid in this.toolbarDefinition) {
        var group = this.toolbarDefinition[groupid];
        for (var itemid in group.items) {
          var item = group.items[itemid];
          if (item.enabled && item.shortcutArray && xopus_globs.utils.matchEventToShortcuts(evt, item.shortcutArray)) {
            //close menus if open
            xopus_globs.contextmenu.hide();

            this.doClickItem(xopus_globs.window.document.getElementById('xopus_toolbar_item_' + itemid), evt);

            //cancel event
            return false;
            break;
          }
        }
      }
    }
    return true;
  },
  symbolMenu:null,
  showInsertSymbolMenu:function() {
    if (!this.symbolMenu) {
      this.symbolMenu = { id:'symbol', showAsMenu:true, showAsGrid:true };
      this.symbolMenu.items = {};
      for (var i=0; i<xopus_globs.symbols.length; i++) {
        this.symbolMenu.items[xopus_globs.symbols[i]] = {
            name:('&#' + xopus_globs.symbols[i] + ';'),
            onclick:function(id) {
              var context = Context.getCurrent();
              var rng = context.xopus.htmldocument.selection.createRange();
              rng.text=String.fromCharCode(1*id);
              rng.collapse(false); 
              rng.select(); 
              xopus_globs.ui.store();
              xopus_globs.ui.notifyChanges(context.xopus);
            }
        };
      }
    }
    xopus_globs.contextmenu.draw(xopus_globs.window, {srcElement:xopus_globs.window.document.getElementById('xopus_toolbar_item_insertsymbol')}, this.symbolMenu);
  },
  doMouseOver: function(evt) {
    //find element
    var el = xopus_globs.browser.ie?evt.srcElement:evt.target;
    if (el.nodeType == 3) el = el.parentNode;

    //see if over item
    if (el.getAttribute('oriclass') == 'xopus_toolbar_item') {
      this.doMouseOverItem(el);
      return false;
    }

    if (el.id == 'xopus_toolbar_xopuslogo') {
      if (el.className == 'xopus_toolbar_xopuslogo') el.className = 'xopus_toolbar_xopuslogo_over';
      return false;
    }

    return true;
  },
  doMouseOut: function(evt) {
    //find element
    var el = xopus_globs.browser.ie?evt.srcElement:evt.target;
    if (el.nodeType == 3) el = el.parentNode;

    //see if out item
    if (el.getAttribute('oriclass') == 'xopus_toolbar_item') {
      this.doMouseOutItem(el);
      return false;
    }

    if (el.id == 'xopus_toolbar_xopuslogo') {
      if (el.className == 'xopus_toolbar_xopuslogo_over') el.className = 'xopus_toolbar_xopuslogo';
      return false;
    }

    return true;
  },
  doClick: function(evt) {
    //find element
    var el = xopus_globs.browser.ie?evt.srcElement:evt.target;
    if (el.nodeType == 3) el = el.parentNode;

    //see if out item
    if (el.getAttribute('oriclass') == 'xopus_toolbar_item') {
      this.doClickItem(el, evt);
      return false;
    }

    return true;
  }
};

numScriptsLoading--;
