////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

function UndoHandler(initialState) 
{
  this.enabled = true;
  
  this.actions = [{actionType: UndoHandler.LOAD, resultState: initialState}];
  this.idx = 0;
}

UndoHandler.prototype.addAction = function(actionType, resultState)
{
  if (!this.enabled)
    return;
  
  this.idx++;
  this.actions[this.idx] = {actionType: actionType, resultState: resultState.cloneNode(true)};  
  this.actions.length = this.idx + 1;
  
  //xopus_globs.utils.debug("adding: " + resultState.xml);
};

UndoHandler.prototype.canUndo = function()
{
  if (!this.enabled)
    return false;
  
  if (Context.hasCurrent() && Context.getCurrent().hasChanged() && !this.actions[this.idx + 1]) return true;
 
  return this.idx > 0 && this.actions.length > 1;
};

UndoHandler.prototype.canRedo = function()
{
  if (!this.enabled)
    return false;

  if (Context.hasCurrent() && Context.getCurrent().hasChanged()) return false;
  
  return this.idx < this.actions.length - 1;
};

UndoHandler.prototype.undo = function()
{
  return this.actions[--this.idx].resultState.cloneNode(true);
};

UndoHandler.prototype.redo = function()
{
  return this.actions[++this.idx].resultState.cloneNode(true);
};

UndoHandler.prototype.getUndoType = function(offset)
{
  if (!offset)
    offset = 0;
  
  if (this.idx - offset < 1)
    return UndoHandler.NULL;
  
  return this.actions[this.idx - offset].actionType;
};

UndoHandler.prototype.getRedoType = function(offset)
{
  if (!offset)
    offset = 0;

  if (this.idx + offset + 1 > this.actions.length - 1)
    return UndoHandler.NULL;
  
  return this.actions[this.idx + offset + 1].actionType;
};

UndoHandler.NULL     = "null";
UndoHandler.LOAD     = "load";
UndoHandler.TYPING   = "typing";
UndoHandler.MOVEUP   = "moveUp";
UndoHandler.MOVEDOWN = "moveDown";
UndoHandler.DELETE   = "delete";    // Delete node
UndoHandler.REMOVE   = "remove";    // Remove node but keep children
UndoHandler.INSERT   = "insert";
UndoHandler.ADDLANG  = "addLang";
UndoHandler.REMLANG  = "remLang";

numScriptsLoading--;