////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

//constructor
function Utils() {}

Utils.prototype={
  cleanupHTML: function (html) {
    // cleans up the given html string:
    // removes unnecessary attributes etc.

    // first clean up <i> tags and <em> tags
    var str=html.replace(/(\<i[^>]*>)/gi, "<i>").replace(/(<\/i>)/gi,"</i>");
    str=str.replace(/(\<em[^>]*>)/gi, "<i>").replace(/(<\/em>)/gi,"</i>");
    // second, clean up <b> and <strong> tags
    str=str.replace(/(\<strong[^>]*>)/gi, "<b>").replace(/(<\/strong>)/gi,"</b>");
    str=str.replace(/(\<b[^>]*>)/gi, "<b>").replace(/(<\/b>)/gi,"</b>");
    return str;

  },

  cleanupXML: function (node)
  {
    var children = node.childNodes;
    for (var i = children.length - 1; i >= 0; i--)
    {
      var child = children[i];

      switch (child.nodeType)
      {
        case 1:
          this.cleanupXML(child);
          break;
        case 3:
          // Remove empty text nodes.
          if (child.nodeValue.length == 0)
          {
            node.removeChild(child);
          }
          // Join adjacent text nodes.
          else if (child.nextSibling && child.nextSibling.nodeType == 3)
          {
            var newTextNode = node.ownerDocument.createTextNode(child.nodeValue + child.nextSibling.nodeValue);
            node.insertBefore(newTextNode, child);
            node.removeChild(child.nextSibling);
            node.removeChild(child);
          }
          break;
        default:
          continue;
      }
    }
  },

  createTextRange: function (commonAncestor, startOffset, endOffset)
  {
    var rng = null;
    var bodyEl = commonAncestor;
    while (bodyEl && (bodyEl.nodeName.toLowerCase() != 'body')) 
      bodyEl = bodyEl.parentNode;
    
    if (bodyEl) 
      rng = bodyEl.createTextRange();
    
    if (xopus_globs.browser.moz)
    {
      rng.mozDOMRange = commonAncestor.ownerDocument.createRange();
      var point = rng.getNodeForOffset(commonAncestor, 0, endOffset);
      rng.mozDOMRange.setEnd(point.node, point.offset);
      var point = rng.getNodeForOffset(commonAncestor, 0, startOffset);
      rng.mozDOMRange.setStart(point.node, point.offset);
    }
    else
    {
      rng.moveToElementText(commonAncestor);
      rng.collapse(true);
      rng.moveEnd("character", endOffset);
      rng.moveStart("character", startOffset);
    }

    return rng;
  },

  countRangeCharacters: function (r)
  {
/*    var rng = r.duplicate();
    var len=0;
    while (rng.compareEndPoints("StartToEnd",rng)==-1)
    {
      rng.moveEnd("character", -1);
      len++;
    }*/
    var rng = r.duplicate();
    var len = rng.text.length;
    rng.collapse();
    var count = len;
    rng.moveEnd("character", len);
    var diff;
    var oldDiff;
    
    // match text length
    while (true)
    {
      diff = len - rng.text.length;
      if (!diff||diff==oldDiff) break;
      oldDiff=diff;
      count += diff;
      rng.collapse();
      rng.moveEnd("character", count);
    }

    // match extra space not in the text property
    while (rng.compareEndPoints("EndToEnd",r)==-1)
    {
      rng.collapse();
      rng.moveEnd("character", count+1);
      if (rng.text.length==len) count++; else break;
    }
    
    return count;
  },

  countSpaceDifference: function (rng)
  {
    var count=0;
    rng.htmlText.replace(/(\&nbsp\;)| /g,function() {count++;return ''});
    rng.text.replace(/ /g,function() {count--;return ''});
    return count;
  },

  /**
   * This function tries to expand the (dom level 2) range as close as possible to the commonAncestorContainer
   * without changing the selected text.
   */
  normalizeRange: function (domRange)
  {
    var container = domRange.commonAncestorContainer;
    
    var ns = domRange.startContainer, is = domRange.startOffset;
    while (ns != container)
    {
      if (is == 0)
        is = this.indexOf(ns);
      else if (is == (ns.length || ns.childNodes.length))
        is = this.indexOf(ns) + 1;
      else
        break;

      ns = ns.parentNode;
    }

    var ne = domRange.endContainer, ie = domRange.endOffset;
    while (ne != container)
    {
      if (ie == 0)
        ie = this.indexOf(ne);
      else if (ie == (ne.length || ne.childNodes.length))
        ie = this.indexOf(ne) + 1;
      else
        break;

      ne = ne.parentNode;
    }

    // While the new selection contains 1 node, select its child.
    while (ns.nodeType == 1 && ne.nodeType == 1 &&
           ns.childNodes[is] == ne.childNodes[ie - 1])
    {
      ns = ns.childNodes[is];
      is = 0;
      ne = ns;
      ie = ne.length || ne.childNodes.length;
    }

    domRange.setEnd(ne, ie);
    domRange.setStart(ns, is);
  },

  getAbsoluteURI: function (baseURI, relURI)
  {
    if (relURI.indexOf("/") == 0 || relURI.indexOf("http://") == 0)
      return relURI;

    if (relURI.indexOf("./") == 0)
      relURI = relURI.substr(2);

    baseURI = baseURI.substring(0, baseURI.lastIndexOf('/'));

    return baseURI + "/" + relURI;
  },

  getPath: function (uri)
  {
    var pos = uri.lastIndexOf('/');

    if (pos != -1)
      return uri.substring(0, pos + 1);
    else
      return uri;
  },

  /**
   * Calculate the an xpath for the given node.
   *
   * For error messages only.
   */
  getXPath: function (node)
  {
    switch (node.nodeType)
    {
      // element
      case 1:
        var localName = this.getLocalName(node);
        var namespaceURI = node.namespaceURI;
        var prefix = node.prefix;
        var idx = node.selectNodes("preceding-sibling::*[local-name() = '" + localName + "' and " +
                                                        "namespace-uri() = '" + namespaceURI + "']").length + 1;
        var parentPath = this.getXPath(node.selectSingleNode("parent::node()"));
        return parentPath + "/" + (prefix?prefix+":":"") + localName + "[" + idx + "]";
      // attribute
      case 2:
        var localName = this.getLocalName(node);
        var prefix = node.prefix;
        var parentPath = this.getXPath(node.selectSingleNode("parent::node()"));
        return parentPath + "/@" + (prefix?prefix+":":"") + localName;
      // text node
      case 3:
        var idx = node.selectNodes("preceding-sibling::text()").length + 1;
        var parentPath = this.getXPath(node.selectSingleNode("parent::node()"));
        return parentPath + "/text()[" + idx + "]";
      // document
      case 9:
      default:
        return "";
    }
  },

  /**
   * Get the local name for the given node.
   */
  getLocalName: function (node)
  {
    if (xopus_globs.browser.moz)
      return node.localName;

    var colonIdx = node.nodeName.indexOf(":");

    if (colonIdx == -1)
      return node.nodeName;
    else
      return node.nodeName.substr(colonIdx + 1);
  },

  /**
   * Get the namespaceURI for the given node.
   */
  getNamespaceURI: function (node)
  {
    return node.namespaceURI || "";
  },

  /**
   * Get the namespace referenced by the given prefix in the given context node.
   */
  getNamespaceForPrefix: function (node, prefix)
  {
    if (xopus_globs.browser.moz)
    {
      var xpe = new XPathEvaluator();
      var nsr = xpe.createNSResolver(node);

      return nsr.lookupNamespaceURI(prefix);
    }
    else
    {
      var namespaces = node.selectNodes("namespace::node()");

      for (var i = 0; i < namespaces.length; i++)
        if ((namespaces[i].nodeName == "xmlns" && prefix == "") ||
            (this.getLocalName(namespaces[i]) == prefix))
          return namespaces[i].nodeValue;

      return null;
    }
  },

  /**
   * Get the shortest valid prefix for the given namespace the given context node or null if none was found.
   */
  getPrefixForNamespace: function (node, namespaceURI)
  {
    if (xopus_globs.browser.moz)
    {
      // TODO implement this

      return null;
    }
    else
    {
      var namespaces = node.selectNodes("namespace::node()");

      var prefix = null;
      for (var i = 0; i < namespaces.length; i++)
      {
        if (namespaces[i].nodeValue == namespaceURI)
        {
          var newPrefix = namespaces[i].nodeName == "xmlns" ? "" : this.getLocalName(namespaces[i]);

          if (!prefix || newPrefix.length < prefix.length)
            prefix = newPrefix;
        }
      }

      return prefix;
    }
  },

  /**
   * Returns a document fragment with the children of the given node.
   */
  getChildren: function(node)
  {
    var doc = node.nodeType == 9 ? node : node.ownerDocument || node.parentNode.ownerDocument;
    var children = doc.createDocumentFragment();

    for (var i = 0; i < node.childNodes.length; i++)
      if (i != 0 || node.nodeType != 9 || node.childNodes[i].nodeType != 7)
        children.appendChild(node.childNodes[i].cloneNode(true));

    return children;
  },

  /**
   * Returns a list of children with the given names in the given namespace.
   */
  getChildrenByName: function(node, names, namespaceURI)
  {
    var children = [];

    var str = "";
    for (var i = 0; i < node.childNodes.length; i++)
    {
      var child = node.childNodes[i];
      var localName = this.getLocalName(child);

      if (names[localName] && child.namespaceURI == namespaceURI)
        children.push(child);
    }

    return children;
  },

  /**
   * Returns the first child with one of the given names in the given namespace.
   */
  getChildByName: function(node, names, namespaceURI)
  {
    var str = "";
    for (var i = 0; i < node.childNodes.length; i++)
    {
      var child = node.childNodes[i];
      var localName = this.getLocalName(child);

      if (names[localName] && child.namespaceURI == namespaceURI)
        return child;
    }

    return null;
  },

  /**
   * Removes all children from a node.
   */
  removeChildren: function (node)
  {
    var children = node.childNodes;
    for (var i = children.length - 1; i >= 0; i--)
      node.removeChild(children[i]);

    return node;
  },

  /**
   * Replace the children of a node.
   */
  replaceChildren: function (node, children)
  {
    this.removeChildren(node);
    node.appendChild(children);

    return node;
  },

  /**
   * Set an attribute with a namespace.
   */
  setAttributeNS: function (parent, namespaceURI, qName, value)
  {
    if (xopus_globs.browser.moz)
    {
      parent.setAttributeNS(namespaceURI, qName, value);
    }
    else
    {
      var attr = parent.ownerDocument.createNode(2, qName, namespaceURI);
      attr.nodeValue = value;
      parent.setAttributeNode(attr);
    }
  },

  /**
   * Set the value of the given attribute node.
   */
  setAttributeValue: function (node, value)
  {
    if (xopus_globs.browser.moz)
    {
      var attr = node.ownerDocument.createNode(2, xopus_globs.utils.getLocalName(node), node.namespaceURI);
      attr.nodeValue = value;

      var elt = node.selectSingleNode("parent::*");
      elt.removeAttributeNode(node);
      elt.setAttributeNode(attr);
    }
    else
    {
      node.nodeValue = value;
    }
  },

  /**
   * Get the index of the given node in its parent.
   */
  indexOf: function (node)
  {
    var i = 0;
    while (node = node.previousSibling)
      i++;

    return i;
  },

  /**
   * See if the first node is an ancestor of the second node.
   *
   * Works for both XML and HTML
   */
  isAncestorOf: function(lhs, rhs)
  {
    for (var elt = rhs; elt != rhs.ownerDocument; elt = elt.parentNode)
      if (elt == lhs)
        return true;

    return false;
  },

  capitalize:function(s) {
    return s.substring(0, 1).toUpperCase() + s.substring(1);
  },
  
  getAnyProperty: function (obj)
  {
    for (var i in obj)
      return i;
  },
  
  objHasProp:function(obj, prop) {
    if (!obj)
      return false;

    var anyProp = false;
    for (var p in obj) {
      anyProp = true;
      if (prop && (p == prop))
        return true;
    }

    if (!prop)
      return anyProp;
    else
      return false;
  },

  shortcutsToArray:function(s) {
    if (!s) return null;
    return s.split(',');
  },
  shortcutsToString:function(s) {
    if (!s) return '';
    s = s.replace(/([^,]*)\,.*/g, "$1");
    s = s.replace(/,\s*/g, ", ");
    s = s.replace(/\#(\d+)/g, function(s, m) {
        m = 1*m;
        var s = '';
        switch (m) {
          case 112: s = 'F1'; break;
          case 33: s = 'PageUp'; break;
          case 34: s = 'PageDown'; break;
          case 38: s = 'ArrowUp'; break;
          case 40: s = 'ArrowDown'; break;
          case 46:
          case 110: s = 'Del'; break;
          case 45:
          case 96: s = 'Ins'; break;
          case 188: s = '<'; break;
          case 190: s = '>'; break;
          default:
            s = String.fromCharCode(m).toUpperCase();
        }
        return s;
      });
    if (xopus_globs.browser.mac) {
      //Mac shift command symbols in reversed order
      s = s.replace(/Ctrl\+Shift\+/g, String.fromCharCode(0x21E7) + String.fromCharCode(0x2318));
      //Mac command symbol (&#u2318)
      s = s.replace(/Ctrl\+/g, String.fromCharCode(0x2318));
      //Mac shift symbol (&#u21E7)
      s = s.replace(/Shift\+/g, String.fromCharCode(0x21E7));
    }
    return " " + "(" + s + ")";
  },
  matchEventToShortcuts:function(evt, shortcutArray) {
    if (!shortcutArray || (shortcutArray.length == 0) || !evt || (!evt.keyCode)) return false;
    var evtString = '' + ((evt.ctrlKey || evt.metaKey)?'Ctrl+':'') + (evt.altKey?'Alt+':'') + (evt.shiftKey?'Shift+':'') + '#' + evt.keyCode;
    for (var i=0; i<shortcutArray.length; i++) if (shortcutArray[i] == evtString) return true;
    return false;
  },
  runHandler:function(item, hndName, param1, param2) {
    if (!item[hndName] && !item.href) return;

    if (item.obj) {
      item.obj[item[hndName]](param1, param2);
    } else if (item[hndName]) {
      item[hndName](param1, param2);
    } if (item.href && (hndName == 'onclick')) {
      window.open(item.href);
    }
  },

  debug: function (str)
  {
    if (xopus_globs.debug) {
      var theHTML = ''+str;
      theHTML = theHTML.replace(/&/g, "&amp;");
      theHTML = theHTML.replace(/</g, "&lt;");
      theHTML = theHTML.replace(/>/g, "&gt;");
      window.open().document.write("<pre>" + theHTML + "\n</pre>");
    }
  }
};

numScriptsLoading--;
