////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

/**
 * XopusUI combines the different GUI compontents.
 */
function XopusUI() 
{
  this.disabled = false;
  
  // Create a list of all buttons having nodes to insert
  this.nodeInsertingButtons = [];
  this.buttonInsertableNodes = {'br':true};
  for (var groupid in xopus_globs.toolbarDefinition)
  {
    var group = xopus_globs.toolbarDefinition[groupid];
    for (var itemid in group.items)
    {
      var item = group.items[itemid];
      item.groupid = groupid;

      if (!item.neverShow && (item.nodes || item.inserttext))
      {
        this.nodeInsertingButtons[this.nodeInsertingButtons.length] = item;

        if (item.nodes)
          for (var i=0; i<item.nodes.length; i++)
            this.buttonInsertableNodes[item.nodes[i].name] = item.nodes[i];
      }
    }
  }
}

XopusUI.prototype =
{
  /**
   * Refresh the preview when new HTML is generated.
   */
  redraw: function(xopus, force)
  {
    this.hideSelection();

    xopus_globs.treemorpher.morph(xopus, force);
    
    // Reanimate all editors if needed.
    if (xopus.canvas.isStarted)
      this.createAll(xopus);
    
    if (!this.selectionMoved)
      this.restoreSelection();
    else
      this.selectionMoved = false;
  },

  /**
   * Attach all editor hooks.
   */
  createAll: function(xopus) {
    // Scan html starting from given element for editable parts
    var elems = xopus.canvas.getElementsByTagName("div");
    
    // This results in a factor 10 performance increase.
    var elemsCopy = {};
    for (var i=0;i<elems.length;i++) 
      elemsCopy[i] = elems[i];
    
    // Initialize the xpath cache.
    var oldXPathCache = xopus.xpathCache;
    xopus.xpathCache = {};
    
    // Attach editing behaviour
    var ed = null;
    var newEd = null;
    for (var i in elemsCopy) 
    {
      var el = elemsCopy[i];
      if (el.getAttribute("xopus_editor") == "true") 
      {
        ed = el;
        // Create a unique id for this editor
        var xpath = el.getAttribute("xpath");
        
        // Check if the referenced node was used as an XInclude.
        var editable = !xopus.pipeline.isXInclude(xpath);
        
        if (!xopus.xpathCache[xpath])
          xopus.xpathCache[xpath] = [];

        var count = xopus.xpathCache[xpath].length;
        xopus.xpathCache[xpath][count] = el;
        
        this.createEditor(xopus, el, xopus.id + ":" + xpath + ":" + count, editable);
        
        // Scroll into view if needed
/*        if (oldXPathCache && !oldXPathCache[xpath])
        {
          newEd = el;
          
          if (editable)
          {
            try {
              var rng = xopus.htmldocument.selection.createRange();
              rng.moveToElementText(el);
              rng.select();
            } catch(e) {}
          }
          else
          {
            el.scrollIntoView();
          }               
      
          oldXPathCache = null;
        }*/
      }
    }
    
    if (newEd)
    {
      // ****** TODO The following should be called in doSelectionChange
      var oldContext = Context.getCurrent();
      
      if (oldContext)
      {
        var oldEditor  = oldContext.getEditor();

        if (oldEditor)
        {
          // Notify the editor
          xopus_globs.editorDriver.stop(oldEditor, oldContext);

          // Notify all synced editors
          this.stopSync();

          // Lose context
          xopus_globs.eventCenter.doContextLose(context);
        }
      }
      
      Context.changeTo(newEd.id, newEd.xopusObject);
      var context = Context.getCurrent();

      // Warn eventcenter the context has changed
      xopus_globs.eventCenter.doContextChange(context);

      // Notify the editor
      if (newEd.editable)
        xopus_globs.editorDriver.start(newEd, context);

      // Notify all synced editors
      this.startSync();

      // *******

      this.selectionMoved = true; 
    }
    
    // Make sure there is at least some context
    if (!Context.hasCurrent() && !Context.hasPrevious()) 
    {
      Context.changeTo(ed.id, xopus);
      Context.lose();
    }
  },

  /**
   * Attach editor hooks to one element.
   */
  createEditor: function(xopus, el, id, editable) {
    
    // check if redraw did not recreate this editor
    if (el.xopusObject) return;
    
    if (el.hasChildNodes && el.childNodes.length == 1 && el.firstChild.nodeName.toLowerCase()=='div')
      el.firstChild.contentEditable = editable;
    else
      el.contentEditable = editable;
    el.editable = editable;
    el.xopusObject = xopus;
    el.id = id;

    xopus_globs.editorDriver.create(el, new Context(id, xopus));
    
    // Check if this editor contains any inline type html elements.
    if (el.children.length != 0 && el.children[0].currentStyle.display == "block")
      el.style.display = "block";
  },

  /**
   * Notify all editors
   */
  destroyAll: function(xopus) 
  {
    // Scan html starting from given element for editable parts
    var elems = xopus.canvas.getElementsByTagName("div");
    
    // Iterate through all editors.
    for (var i=0; i<elems.length; i++)
    {
      var el = elems[i];
      if (el.xopusObject) 
        xopus_globs.editorDriver.destroy(el, new Context(el.id, xopus));
    }
  },

  /**
   * The keyup hook for an editor.
   */
  doKeyUpEditor: function(evt)
  {
    //check for current context
    var curContext = Context.getCurrent();
    if (!curContext) return true;

    //find the element that generated the event
    var elt = curContext.getEditor();

    //check the element and correct a redraw problem
    //check for backspace/delete
    if ((evt.keyCode == 8) || (evt.keyCode == 46)) {
        //workaround for editable areas that shrink because of delete actions but do not resize properly
        //force a redraw
        elt.style.display = "block";
        elt.style.display = "inline";
    }

    //sync changes between linked editors
    this.doSync();

//    if (!xopus_globs.curHasChanged || !curContext.xopus.hasChanged)
    if (curContext.hasChanged())
      this.notifyChanges(curContext.xopus);
  },

  // Get a list of editors sorted on the given side.
  getMapping: function(side)
  {
    // The mapping
    var mapping = [];

    // Scan html starting from given element for editable parts
    var elems = Context.getCurrent().xopus.canvas.getElementsByTagName("div");

    // Attach editing behaviour
    for (var i = 0; i < elems.length; i++)
    {
      var el = elems[i];

      if (el.getAttribute("xopus_editor") == "true")
      {
        // Store x and y coordinates.
        var rect = el.getBoundingClientRect();
        var map = { r: rect, val: el };
        mapping.push(map);
      }
    }

    switch (side)
    {
      case "left":
        return mapping.sort(function(a, b) {return a.r.left   - b.r.left;});
      case "right":
        return mapping.sort(function(a, b) {return a.r.right  - b.r.right;});
      case "top":
        return mapping.sort(function(a, b) {return a.r.top    - b.r.top;});
      case "bottom":
        return mapping.sort(function(a, b) {return a.r.bottom - b.r.bottom;});
    }
  },

  /**
   * The keypress hook.
   */
  doKeyDownEditor: function (evt)
  {
    var elt = evt.srcElement;

    //leave untouched if not editable
    if (!elt || !elt.isContentEditable) return true;

    //find context
    var context = Context.getCurrent();

    //leave if no context
    if (!context) return true;

    //find the document
    var theDocument = context.xopus.htmldocument;
    if (!theDocument) return true;

    // Calculate the distance between a range and a point.
    var calcDist = function(lower, upper, pos)
    {
      var lDist = lower - pos;
      var uDist = upper - pos;
      return Math.min(Math.abs(lDist), Math.abs(uDist)) * (lDist < 0 ? -1 : 1) * (uDist < 0 ? -1 : 1);
    };

    var result = true;
    switch (evt.keyCode)
    {
      case 13:
        if (xopus_globs.browser.moz) {
          if (!evt.shiftKey) {
            //insert <p /> or <li />
__xopus_alert('insert p or li');
          } else {
            //insert <br />
__xopus_alert('insert br');
          }
          result = false;
        } else {

          if (!evt.shiftKey) {
            //enter: cancel the key if a P nor a LI are allowed in here
            
            //find the xml node belonging to the current editor
            var edNode = context.getEditorElement();
            
            context.xopus.handleChanges(context.getEditor());

            //flag whether allowed
            var allowed = false;
            
            //find all ancestors
            var ancestors = context.getAncestors();

            //loop all ancestors
            var startI = ancestors.length - 1;
            for (var i = startI; i>=0; i--) {
              var children = context.getPossibleChildren(ancestors[i]);

              if (children.p || children.li) {
                allowed = true;
                break;
              }

              //stop if the editor node has been reached
              if (ancestors[i] == edNode) break;
            }
            result = allowed;
          } else {
            //shift-enter: try to insert a BR, if allowed
            var posChildren = context.getPossibleChildren(context.getSelectionNode());

            if (!posChildren.br) result = false;
          }
        }
        break;
      case 65:
        if (evt.ctrlKey) {
          //select all (Ctrl-A)
          //handle select all differently for td and li elements

          //get the current selection
          var rng = context.xopus.htmldocument.selection.createRange();
          if (!rng.parentElement) return true;

          //find the element the selection is in and make sure it is editable
          var el = rng.parentElement();
          if (!el.isContentEditable) return true;

          //find whether this is inside a td or li element (and still editable)
          if (el.nodeType == 3) el = el.parentNode;
          while (el && el.isContentEditable && (el.nodeName.toLowerCase() != "td") && (el.nodeName.toLowerCase() != "li")) el = el.parentNode;
          if (el && el.isContentEditable) {
            //valid td or li element
        
            //select the compete text of the td or li
            rng.moveToElementText(el);
            rng.select();
            result = false;
          } else {
            //no valid td or li element, let the event pass
            result = true;
          }
        }
        break;
      case 83: 
        if (evt.ctrlKey && evt.altKey) 
        { //Ctrl-Alt-S
          //debug function: show the html and xml source of the current editor
          var editor = context.getEditor();
          if (!editor) break;
          
          xopus_globs.utils.debug(
            "OuterHTML of current editor:\n"+editor.outerHTML+"\n\n"+
            "xml of current editor:\n"+context.getEditorNode().xml
          );
          
          result = false;
        }
        break;
      case 66:
        //bold (Ctrl-B)
      case 73:
        //italic (Ctrl-I)
      case 75:
        //link (Ctrl-K)
      case 77:
        //??? (Ctrl-M)
      case 85:
        //underline (Ctrl-U)
        if (evt.ctrlKey) result = false;
        break;
      case 38:
//TODO: repair the below code... if on it will stop up-moves in multi-line fields
/*
        // Up arrow
        if (this.cursorReachedSide(elt, "top")) {
          var range = theDocument.selection.createRange();
          var x = range.boundingLeft + range.boundingWidth / 2;

          // Get a sorted list of editors.
          var m = this.getMapping("bottom");

          // Search first element with a bottom above the cursor top.
          var prev;
          var idx = -1;
          for (i = 0; i < m.length && m[i].r.bottom < range.boundingTop; i++)
            prev = m[idx = i].val;

          // Search through all elements on the same 'row' for the best x match.
          var dist = 0xFFFF;
          for (var j = idx; j >= 0 && m[idx].r.top < m[j].r.bottom; j--)
          {
            var currDist = calcDist(m[j].r.left, m[j].r.right, x);
            if (currDist < dist)
            {
              prev = m[j].val;
              dist = currDist;
            }
          }

          if (prev)
          {
            var r = prev.getBoundingClientRect();
            var newX = x <= r.left ? r.left + 1 : x >= r.right ? r.right - 1 : x;
            var newY = r.bottom - 1;

            range.moveToPoint(newX, newY);
            range.select();

            // Cancel cursor move if we skipped a text node.
            if (prev != elt.previousSibling)
              evt.returnValue = false;
          }
        }
        result = false;
*/
        break;
      case 40:
//TODO: repair the below code... if on it will stop down-moves in multi-line fields
/*
        // Down arrow
        if (this.cursorReachedSide(elt, "bottom")) {
          var range = theDocument.selection.createRange();
          var x = range.boundingLeft + range.boundingWidth / 2;

          // Get a sorted list of editors.
          var m = this.getMapping("top");

          // Search first element with a top below the cursor bottom.
          var next;
          var idx = m.length + 1;
          for (i = m.length - 1; i >= 0 && m[i].r.top > range.boundingTop + range.boundingHeight; i--)
            next = m[idx = i].val;

          // Search through all elements on the same 'row' for the best x match.
          var dist = 0xFFFF;
          for (var j = idx; j < m.length && m[idx].r.bottom > m[j].r.top; j++)
          {
            var currDist = calcDist(m[j].r.left, m[j].r.right, x);
            if (currDist < dist)
            {
              next = m[j].val;
              dist = currDist;
            }
          }

          if (next)
          {
            var r = next.getBoundingClientRect();
            var newX = x <= r.left ? r.left + 1 : x >= r.right ? r.right - 1 : x;
            var newY = r.top + 1;

            range.moveToPoint(newX, newY);
            range.select();

            // Cancel cursor move if we skipped a text node.
            if (next != elt.previousSibling)
              evt.returnValue = false;
          }
        }

        result = false;
*/
        break;
      case 37:
//TODO: repair the below code... if on it will fail
/*
        // Left arrow
        if (this.cursorReachedSide(elt, "left")) {
          var range = theDocument.selection.createRange();
          var y = range.boundingTop + range.boundingHeight / 2;

          // Get a sorted list of editors.
          var m = this.getMapping("right");

          // Search first element with a bottom above the cursor top.
          var prev;
          var idx = -1;
          for (i = 0; i < m.length && m[i].r.right < range.boundingLeft; i++)
            prev = m[idx = i].val;

          // Search through all elements on the same 'row' for the best x match.
          var dist = 0xFFFF;
          for (var j = idx; j >= 0 && m[idx].r.left < m[j].r.right; j--)
          {
            var currDist = calcDist(m[j].r.top, m[j].r.bottom, y);
            if (currDist < dist)
            {
              prev = m[j].val;
              dist = currDist;
            }
          }

          if (prev)
          {
            var r = prev.getBoundingClientRect();
            var newX = r.right - 1;
            var newY = y <= r.top ? r.top + 1 : y >= r.bottom ? r.bottom - 1: y;

            range.moveToPoint(newX, newY);
            range.select();

            // Cancel cursor move if we skipped a text node.
            if (prev != elt.previousSibling)
              evt.returnValue = false;
          }
        }

        result = false;
*/
        break;
      case 8:
        // Backspace
//TODO: repair the below code... if on it will fail
/*
        if (this.cursorReachedSide(elt, "left"))
        {
          var prev = elt.previousSibling;
          if (prev && prev.contentEditable) {
            var range = theDocument.selection.createRange();
            range.moveToElementText(prev);
            range.collapse(false);
            range.select();
            range.expand("character");
          }
        }

        elt.style.display = "block";
        elt.style.display = "inline";

        result = false;
*/
        break;
      // Right arrow
      case 39:
//TODO: repair the below code... if on it will fail
/*
        if (this.cursorReachedSide(elt, "right")) {
          var range = theDocument.selection.createRange();
          var y = range.boundingTop + range.boundingHeight / 2;

          // Get a sorted list of editors.
          var m = this.getMapping("left");

          // Search first element with a top below the cursor bottom.
          var next;
          var idx = m.length + 1;
          for (i = m.length - 1; i >= 0 && m[i].r.left > range.boundingLeft + range.boundingWidth; i--)
            next = m[idx = i].val;

          // Search through all elements on the same 'row' for the best x match.
          var dist = 0xFFFF;
          for (var j = idx; j < m.length && m[idx].r.right > m[j].r.left; j++)
          {
            var currDist = calcDist(m[j].r.top, m[j].r.bottom, y);
            if (currDist < dist)
            {
              next = m[j].val;
              dist = currDist;
            }
          }

          if (next)
          {
            var r = next.getBoundingClientRect();
            var newX = r.left + 1;
            var newY = y <= r.top ? r.top + 1 : y >= r.bottom ? r.bottom - 1: y;

            range.moveToPoint(newX, newY);
            range.select();

            // Cancel cursor move if we skipped a text node.
            if (next != elt.previousSibling)
              evt.returnValue = false;
          }
        }

        result = false;
*/
        break;
      case 46:
      /*
        // Delete
        if (this.cursorReachedSide(elt, "right"))
        {
          var next = elt.nextSibling;
          if (next && next.contentEditable) {
            var range = theDocument.selection.createRange();
            range.moveToElementText(next);
            range.collapse(true);
            range.select();
            range.expand("character");
          }
        }

        elt.style.display = "block";
        elt.style.display = "inline";

        result = false;
        */
        break;
    }
    
    return result;
  },

  /**
   * Check if the cursor has reached the left or right side of the given element.
   */
  cursorReachedSide: function (element, side)
  {
    if (xopus_globs.browser.moz) return;

    var theDocument = Context.getCurrent().xopus.htmldocument;
    if (!theDocument) return;

    switch (side)
    {
      case "top":
      case "bottom":
        var range = theDocument.selection.createRange();
        var range2 = range.duplicate();
        range2.moveToElementText(element);
        return (side == "top" && range2.boundingTop == range.boundingTop) ||
               (side == "bottom" && range2.boundingTop + range2.boundingHeight == range.boundingTop + range.boundingHeight);
      case "right":
      case "left":
        var range = theDocument.selection.createRange();
        var range2 = range.duplicate();
        range2.moveToElementText(element);
        range2.collapse(side == "left");

        return range.isEqual(range2);
    }
  },

  /**
   * Send sync events to all editors containing the same node.
   */
  startSync: function ()
  {
    var context = Context.getCurrent();
    var elt = context.getEditor();
    var xpath = elt.getAttribute("xpath");

    for (var i = 0; i < context.xopus.xpathCache[xpath].length; i++)
    {
      if (context.xopus.xpathCache[xpath][i] != elt)
        xopus_globs.editorDriver.sync(context.xopus.xpathCache[xpath][i], context);
    }
  },


  doSync: function ()
  {
    // Check for changes
    var curContext = Context.getCurrent();
    if (curContext.hasChanged())
    {
      var editorEl = curContext.getEditor();
      var xpath = editorEl.getAttribute("xpath");
      var theXPathCache = curContext.xopus.xpathCache[xpath];

      for (var i=0; i<theXPathCache.length; i++)
      {
        if (theXPathCache[i] != editorEl)
          xopus_globs.editorDriver.update(theXPathCache[i], curContext, editorEl.innerHTML);
      }
    }
  },

  /**
   * Send unsync events to all editors containing the same node.
   */
  stopSync: function ()
  {
    var context = Context.getCurrent();
    if (!context) return;

    var elt = context.getEditor();
    var xpath = elt.getAttribute("xpath");

    for (var i = 0; i < context.xopus.xpathCache[xpath].length; i++)
    {
      if (context.xopus.xpathCache[xpath][i] != elt)
        xopus_globs.editorDriver.unsync(context.xopus.xpathCache[xpath][i], context);
    }
  },

  /**
   * The mouseover hook.
   */
  doMouseOver: function (evt, theWindow)
  {
    if (this.disabled) return true;
    
    var el = xopus_globs.browser.ie?evt.srcElement:evt.target;
    if (el.nodeType == 3) el = el.parentNode;
    while (el && !el.xopusObject) el = el.parentNode;
    if (!el) return true;

    var context = new Context(el.getAttribute("id"), el.xopusObject);
    
    xopus_globs.toolbar.setStatusValue(context.toString());

    // Mozilla workaround!
    var restoreSelection = false;
    if (xopus_globs.browser.moz && Context.getCurrent()) {
      // Store the selection in order to put it back later
      // mozilla will otherwise hide the current selection because the editorDriver changes someone elses className (sounds logical doesn't it?)
      this.storeSelection();
      if (this.selLen > 0) restoreSelection = true;
    }
    
    xopus_globs.editorDriver.mouseOver(el, context);

    // Mozilla workaround!
    if (restoreSelection) {
      //restore the selection now
      this.restoreSelection();
    }
  
    //cancel the event
    xopus_globs.framework.cancelEvent(evt);
    return false;
  },

  /**
   * The mouseout hook.
   */
  doMouseOut: function (evt, theWindow)
  {
    if (this.disabled) return true;

    var el = xopus_globs.browser.ie?evt.srcElement:evt.target;
    if (el.nodeType == 3) el = el.parentNode;
    while (el && !el.xopusObject) el = el.parentNode;
    if (!el) return true;

    var context = new Context(el.getAttribute("id"), el.xopusObject);
    
    xopus_globs.toolbar.resetStatusValue();
    xopus_globs.editorDriver.mouseOut(el, context);

    //cancel the event
    xopus_globs.framework.cancelEvent(evt);
    return false;
  },

  /**
   * The selectionchange hook
   *  this routine should check the current selection and compare it to the previous selection
   *
   *  if the parent-editor has changed the current editor should change
   *  if the parent-node-context has changed buttons should be re-evaluated
   */
  doSelectionChange: function (evt, theWindow)
  {
var timeIt = false;
var status = '';
    
    if (this.disabled) return;

    // Events that originated in the xopus window can never make the selection change
    if (theWindow == xopus_globs.window) 
      return;

/*
    // Don't accept a new selection if it is not inside an editable region
    var newEditor = xopus_globs.browser.ie ? evt.srcElement : evt.target;

    // Events that originated from the textarea don't count
    if (newEditor && newEditor.id == 'xopus_textarea')
      return;
*/

    var rng = this.fixSelection(theWindow);
    
    // Derive current editor from selection
    var newEditor = rng.parentElement();
    
if (timeIt) var st = new Date();
    // Find the editor html element
    while (newEditor && !newEditor.xopusObject) 
      newEditor = newEditor.parentNode;
    
    // Get the current context
    var context = Context.getCurrent();
if (timeIt) status = "1: " + (new Date() - st);

    //check if really inside editor
    if (newEditor && (evt.type == 'mouseup')) {
      var bcr = newEditor.getBoundingClientRect();
      var x = evt.clientX;
      var y = evt.clientY;
      if (
        (x < bcr.left) 
        || (x > bcr.right)
        || (y < bcr.top) 
        || (y > bcr.bottom)
      ) newEditor = null;
    }
    
    // Check if the selection moved 
    // from an editor outside all editors
    // of from outside all editors to an editor
    // or to another editor
    if ((!newEditor && context) || (!context && newEditor) || (context && newEditor && newEditor.id != context.id)) 
    {
      // Is there a current editor?
      if (context) 
      {
if (timeIt) var st = new Date();
        // Notify the editor
        xopus_globs.editorDriver.stop(context.getEditor(), context);

        // Notify all synced editors
        this.stopSync();

        // Lose context
        xopus_globs.eventCenter.doContextLose(context);

        // Store changes
        this.store();

        Context.lose();
if (timeIt) status += " 2: " + (new Date() - st);
      }
      
      // Is there a new editor?
      if (newEditor) 
      {
if (timeIt) var st = new Date();
        // Change the context.
        Context.changeTo(newEditor.id, newEditor.xopusObject);
        context = Context.getCurrent();

        // Warn eventcenter the context has changed
        xopus_globs.eventCenter.doContextChange(context);

        // Notify the editor
        xopus_globs.editorDriver.start(newEditor, context);

        // Notify all synced editors
        this.startSync();
if (timeIt) status += " 3: " + (new Date() - st);
      }
      
if (timeIt) var st = new Date();
      // Redraw directly
      if (this.redrawTimerId || context.redrawRequested()) 
      {
        clearTimeout(this.redrawTimerId);
        this.redrawTimerId = 0;
        
        this.redraw(context.xopus);
      }
if (timeIt) status += " 4: " + (new Date() - st);

if (timeIt) var st = new Date();
      // Set button states if needed.
      this.setButtonState();
if (timeIt) status += " 5: " + (new Date() - st);
    }

    // Set the cut/copy/paste buttons
    if (xopus_globs.browser.ie) {
if (timeIt) var st = new Date();
      var isEditable = newEditor && newEditor.editable;
      var rng = theWindow.document.selection.createRange();
      var textSelected = rng && rng.htmlText && (rng.htmlText.length > 0);
      var theDoc = theWindow.document;
      
      xopus_globs.toolbar.setEnabled('edit', 'cut', context && (isEditable && textSelected && theDoc.queryCommandEnabled('cut')));
      xopus_globs.toolbar.setEnabled('edit', 'copy', context && (textSelected && theDoc.queryCommandEnabled('copy')));
      xopus_globs.toolbar.setEnabled('edit', 'paste', context && (isEditable && theDoc.queryCommandEnabled('paste')));
if (timeIt) status += " 6: " + (new Date() - st);
    }
    window.status = status;
  },


  /**
   * Store the changes in the current editor.
   */
  store: function() 
  {
    // See if there are changes.
    var context = Context.getCurrent();
    if (!context || !context.hasChanged()) 
      return;

    // Store the changes if needed
    var editor = context.getEditor();
    if (editor.editable) 
      editor.xopusObject.handleChanges(editor);
    
    // Redraw
    if (this.redrawTimerId) 
      clearTimeout(this.redrawTimerId);

    var myself = this;
    this.redrawTimerId = setTimeout(function (){myself.redraw(editor.xopusObject)}, 0);
  },
  
  /**
   * Notifies globs that the current XML has changed.
   */
  notifyChanges: function(xopus)
  {
    if (Context.hasCurrent()) {
      this.doSync();

      // (De)activate undo/redo
      xopus_globs.set("toolbarDefinition.undo.items.undo.enabled", xopus.undohandler.canUndo());
      xopus_globs.set("toolbarDefinition.undo.items.redo.enabled", xopus.undohandler.canRedo());
    }
    if (!xopus_globs.curHasChanged) xopus_globs.set('curHasChanged', true);
    if (!xopus_globs.anyHasChanged) xopus_globs.set('anyHasChanged', true);
    if (!xopus.hasChanged) xopus.hasChanged = true;
  },

  /**
   * Save the current XML.
   */
  saveCurrent: function()
  {
    var context = Context.getContext();
    if (!context) return;
    
    this.saveOne(context.xopus);
  },
  reloadCurrent: function()
  {
    var context = Context.getContext();
    if (!context) return;


    xopus_globs.dialog.alert(
      xopus_globs.locale.getString('reload?', {}),
      null,
      function(){
        var context = Context.getContext();
        if (!context) return;

        var xopus = context.xopus;
        xopus.reload();
      },
      function(){}
    );
  },
  saveAll:function() {
    for (var id in xopus_globs.xopi) {
      this.saveOne(xopus_globs.xopi[id]);
    }
  },
  saveOne:function(xopus) {
    if (!xopus) return;

    var context = Context.getCurrent();
    if (context && (context.xopus == xopus)) {
      xopus_globs.ui.store();
    }

    if (!context)
      context = Context.getPrevious();
    
    if (xopus.hasChanged) {
      xopus.save();
      xopus.hasChanged = false;
    }

    var hasChanged = false;
    for (var id in xopus_globs.xopi) {
      hasChanged = hasChanged || xopus_globs.xopi[id].hasChanged;
    }
    xopus_globs.set('anyHasChanged', hasChanged);
    if (context.xopus == xopus) xopus_globs.set('curHasChanged', false);
  },

  /**
   * Create move up menu.
   */
  showMoveUpMenu: function(evt, buttonEl, dontMove)
  {
    var context = Context.getCurrent();
    var xopusType = (context.getEditor().getAttribute('xopus_type') != "element");

    var menu = {id:'moveupnode', items:{}, showAsMenu: true};
    
    // Store changes
    xopus_globs.ui.store();
    
    var ancestors = context.getAncestors();
    //var startI = (xopusType?(ancestors.length-1):(ancestors.length-2));
    var startI = ancestors.length-1;
    for (var i = startI; i>0; i--)
    {
      var node = ancestors[i];

      //do not add children that can be manipulated using buttons
      if (this.buttonInsertableNodes[node.nodeName]) continue;

      var enabled = context.canMoveUp(node);

      menu.items[i + "Move " + node.nodeName + " up"] = {
        name: xopus_globs.locale.getString('moveupnodeitem', {'node':context.getPrettyName(node)}),
        disabled:!enabled,
        onclick:(enabled?function()
        {
          var c = Context.getCurrent();
          c.xopus.handleChanges(c.getEditor());
          c.xopus.moveUp(this.params.node);
          xopus_globs.ui.notifyChanges(c.xopus);
          xopus_globs.ui.setButtonState();
          if (c.canMoveUp(this.params.node))
            xopus_globs.ui.showMoveUpMenu(evt, buttonEl, true);
          else
            xopus_globs.contextmenu.hide();
        }:null),
        params: {node : node}
      };

      //separator after current node
      if (i >= startI) menu.items["separator" + i] = {};
    }
    xopus_globs.contextmenu.draw(xopus_globs.window, {type:evt.type, srcElement:buttonEl}, menu, true, dontMove);
  },

  /**
   * Create move down menu.
   */
  showMoveDownMenu: function(evt, buttonEl, dontMove)
  {
    var context = Context.getCurrent();
    var xopusType = (context.getEditor().getAttribute('xopus_type') != "element");

    var menu = {id:'movedownnode', items:{}, showAsMenu: true};

    // Store changes
    xopus_globs.ui.store();
    
    var ancestors = context.getAncestors();
    //var startI = (xopusType?(ancestors.length-1):(ancestors.length-2));
    var startI = ancestors.length-1;
    for (var i = startI; i>0; i--)
    {
      var node = ancestors[i];

      //do not add children that can be manipulated using buttons
      if (this.buttonInsertableNodes[node.nodeName]) continue;

      var enabled = context.canMoveDown(node);

      menu.items[i + "Move " + node.nodeName + " down"] = {
        name: xopus_globs.locale.getString('movedownnodeitem', {'node': context.getPrettyName(node)}),
        disabled:!enabled,
        onclick:(enabled?function()
        {
          var c = Context.getCurrent();
          c.xopus.handleChanges(c.getEditor());
          c.xopus.moveDown(this.params.node);
          xopus_globs.ui.notifyChanges(c.xopus);
          xopus_globs.ui.setButtonState();
          if (c.canMoveDown(this.params.node))
            xopus_globs.ui.showMoveDownMenu(evt, buttonEl, true);
          else
            xopus_globs.contextmenu.hide();
        }:null),
        params: {node : node}
      };

      //separator after current node
      if (i >= startI) menu.items["separator" + i] = {};
    }

    // Determine preferred alignment for the menu
    this.setMenuAlignment(menu);

    xopus_globs.contextmenu.draw(xopus_globs.window, {type:evt.type, srcElement:buttonEl}, menu, true, dontMove);
  },

  /**
   * Create delete menu.
   */
  showDeleteMenu: function(evt, buttonEl)
  {
    var context = Context.getCurrent();
    var xopusType = (context.getEditor().getAttribute('xopus_type') != "element");

    var menu = {id:'deletenode', items:{}, showAsMenu: true};
    var fun = function()
    {
      var xopus = Context.getCurrent().xopus;
      xopus_globs.eventCenter.doContextLose(Context.getCurrent());
      Context.lose();
      xopus.deleteNode(this.params.node);
      xopus_globs.ui.notifyChanges(xopus);
      xopus_globs.ui.setButtonState();
    };

    var ancestors = context.getAncestors();
    //var startI = (xopusType?(ancestors.length-1):(ancestors.length-2));
    var startI = ancestors.length-1;
    for (var i = startI; i>0; i--)
    {
      var node = ancestors[i];

//TODO: temporarily disabled to let users delete lists and such...
//maybe permanent...
//      //do not add children that can be deleted using buttons
//      if (this.buttonInsertableNodes[node.nodeName]) continue;

      var canDelete = context.canDelete(node);
      menu.items[i + "Delete " + node.nodeName] = {
        name: xopus_globs.locale.getString('deletenodeitem', {'node':context.getPrettyName(node)}),
        disabled:!canDelete,
        onclick:(canDelete?fun:null),
        params: {node: node}};

      //separator after current node
      if (i >= startI) menu.items["separator" + i] = {};
    }

    // Add the current node if it is an attribute.
    var node = Context.getCurrent().getEditorNode();
    if (node.nodeType == 2)
    {
      if (context.canDelete(node))
        menu.items["Delete " + node.nodeName] = {
          name: xopus_globs.locale.getString('deletenodeitem', {'node':context.getPrettyName(node)}),
          onclick: fun,
          params: {node: node}};
    }

    // Determine preferred alignment for the menu
    this.setMenuAlignment(menu);

    xopus_globs.contextmenu.draw(xopus_globs.window, {type:evt.type, srcElement:buttonEl}, menu);
  },

  /**
   * Create insert menu.
   */
  showInsertMenu: function(evt, buttonEl)
  {
    var context = Context.getCurrent();
    var xopusType = (context.getEditor().getAttribute('xopus_type') != "element");

    var menu = {id:'insertnode', items:{}, showAsMenu: true};

    var ancestors = context.getAncestors();
    //var startI = (xopusType?(ancestors.length-1):(ancestors.length-2));
    var startI = ancestors.length-1;
    for (var i = startI; i>=0; i--)
    {
      var children = context.getPossibleChildren(ancestors[i]);

      var subMenu = null;
      var theItems = null;
      if (i == startI) {
        theItems = menu.items;
      } else {
        subMenu = { items:{} };
        theItems = subMenu.items;
      }

      var hasChildren = false;
      for (var child in children)
      {
        //do not add children that can be inserted using buttons
        if (this.buttonInsertableNodes[child]) continue;

        hasChildren = true;

        theItems[i + "Add " + child + " to " + ancestors[i].nodeName] = {
          name: xopus_globs.locale.getString('insertnodeitem', {'child':context.getPrettyNameFromDeclaration(children[child]), 'parent': context.getPrettyName(ancestors[i])}),
          onclick: function() {
            var context = Context.getCurrent();
            xopus = context.xopus;
            xopus.handleChanges(context.getEditor());
            xopus.insertNode(this.params.decl, this.params.parent, this.params.refNode);
            xopus_globs.ui.notifyChanges(xopus);
            xopus_globs.ui.setButtonState();
          },
          params: { decl: children[child], parent: ancestors[i], refNode: ancestors[i + 1]}};
      }

      if (i == startI) {
      } else {
        menu.items[i + "Add to " + ancestors[i].nodeName] = {
          name:xopus_globs.locale.getString('addto', {node: context.getPrettyName(ancestors[i])}),
          menu:(hasChildren?subMenu:null),
          disabled:!hasChildren
        };
        var theItems = subMenu.items;
      }

      //separator after current node
      if (i == startI) menu.items["separator" + i] = {};
    }

    // Determine preferred alignment for the menu
    this.setMenuAlignment(menu);

    xopus_globs.contextmenu.draw(xopus_globs.window, {type:evt.type, srcElement:buttonEl}, menu);
  },

  
  // *********** Rich text editing functions *********** //

  insertNodeAround: function(nodes, mustInsert, activeName)
  {
    var context = Context.getCurrent();
    var xopus = context.xopus;

    //find the selection
    var rng = xopus.htmldocument.selection.createRange();

    //if selection is empty try to select a word
    if (rng.text.length == 0)
      rng.expand("word");
    
    //if selection only contains whitespace insert pretty name
    if (rng.text.replace(/\s|\u00A0/g, "").length == 0)
    {
      var pName = '...';
      if (activeName) {
        var prettyDefault = xopus_globs.locale.getString('pretty:' + activeName);
        if (prettyDefault != 'pretty:' + activeName) pName = '[' + xopus_globs.locale.getString('pretty:' + activeName) + ']';
      }

      rng.collapse();
      rng.text = pName;
      rng.moveStart("character", -pName.length);
      rng.select();
    }

    xopus.insertNodeAround(nodes, mustInsert);
    this.notifyChanges(xopus);
    this.setButtonState();
  },

  insertNodeAroundIE: function(command, ieNodeName, actualNodeName)
  {
    var context = Context.getCurrent();
    var xopus = context.xopus;
    var editor = context.getEditor();

    xopus.htmldocument.execCommand(command);
    if (actualNodeName != ieNodeName) {
      var htmlElementTranslation = {};
      htmlElementTranslation[ieNodeName] = actualNodeName;
      xopus.handleChanges(editor, false, htmlElementTranslation);
    }

    this.notifyChanges(xopus);
    this.setButtonState();
  },

  /**
   * Xopus is ready.
   */
  ready: function()
  {
    window.status = xopus_globs.locale.getString("done");
  },

  /**
   * Disable the UI. Probably because a fatal error occured.
   */
  disable: function()
  {
    this.disabled = true;

    window.status = xopus_globs.locale.getString("disabled");
  },

  /**
   * Set the state of the buttons in the toolbar.
   */
  setButtonState: function()
  {
    var canUndo       = false;
    var canRedo       = false;
    var curHasChanged = false;
    var buttonStates;
    
    var context = Context.getCurrent();
    if (context)
    {
      // Get the current selection node. 
      var selNode = context.getSelectionNode();
      var selXPath = context.xopus.pipeline.getXPath(selNode);
      
      // Check the cache.
      buttonStates = context.xopus.cache.get(selXPath, "1");
      if (!buttonStates)
      {
        buttonStates =
        {
          canMoveUp:   false,
          canMoveDown: false,
          canDelete:   false,
          canCreate:   false,
          items:       []
        };
        
        var ancestors = context.getAncestors();
        for (var i = ancestors.length - 1; i >= 0; i--)
        {
          var node = ancestors[i];

          if (!xopus_globs.toolbar.toolbarDefinition.node.items.moveupnode.neverShow && !buttonStates.canMoveUp && context.canMoveUp(node)) 
            buttonStates.canMoveUp = true;

          if (!xopus_globs.toolbar.toolbarDefinition.node.items.movedownnode.neverShow && !buttonStates.canMoveDown && context.canMoveDown(node)) 
            buttonStates.canMoveDown = true;

          if (!buttonStates.canDelete && context.canDelete(node)) 
            buttonStates.canDelete = true;

          if (!buttonStates.canCreate) 
          {
            var children = context.getPossibleChildren(node);
            for (var c in children) 
            {
              buttonStates.canCreate = true;
              break;
            }
          }

          if (buttonStates.canMoveUp && buttonStates.canMoveDown && buttonStates.canDelete && buttonStates.canCreate) 
            break;
        }
        
        // Get the current selection element. 
        var selElt = selNode;
        if (selElt.nodeType == 3)
          selElt = selElt.parentNode;
        
        // Get the current editor element
        var edElt = context.getEditorElement();
        
        // Activate all appropriate node inserting buttons
        for (var i = 0; i < this.nodeInsertingButtons.length; i++) 
        {
          var item = this.nodeInsertingButtons[i];
          var itemActive = false;

          // Activate buttons that want to insert text
          if (item.inserttext)
            itemActive = true;

          // Activate buttons that want to insert nodes
          for (var n = selElt; !itemActive  && item.nodes && n != edElt.parentNode; n = n.parentNode)
            for (var j = 0; j < item.nodes.length; j++)
            {
              itemActive = context.canContainNode(n, item.nodes[j].name, item.nodes[j].namespace);
              if (itemActive)
              {
                item.activeName = item.nodes[j].name;
                break;
              }
            }
            
          
          buttonStates.items.push(
            {
              name: "toolbarDefinition." + item.groupid + ".items." + item.id + ".enabled", 
              value: itemActive
            });
        }
        
        // Store in cache.
        context.xopus.cache.put(selXPath, "1", buttonStates);
      }

      // Record save state
      curHasChanged = context.xopus.hasChanged;
    }
    else
    {
      buttonStates =
      {
        canMoveUp:   false,
        canMoveDown: false,
        canDelete:   false,
        canCreate:   false,
        items:       []
      };
      
      // Record save state
      curHasChanged = Context.getPrevious().xopus.hasChanged;
      
      // Deactivate all node inserting buttons
      for (var i = 0; i < this.nodeInsertingButtons.length; i++) 
      {
        var item = this.nodeInsertingButtons[i];

        if (item.nodes || item.inserttext)
          buttonStates.items.push(
            {
              name: "toolbarDefinition." + item.groupid + ".items." + item.id + ".enabled", 
              value: false
            });
      }    
    }

    // Set save icon by setting the changed state
    if (xopus_globs.curHasChanged != curHasChanged) 
      xopus_globs.set('curHasChanged', curHasChanged);

    // Set structure menus
    if (!xopus_globs.toolbar.toolbarDefinition.node.items.moveupnode.neverShow) xopus_globs.set("toolbarDefinition.node.items.moveupnode.enabled",   buttonStates.canMoveUp);
    if (!xopus_globs.toolbar.toolbarDefinition.node.items.movedownnode.neverShow) xopus_globs.set("toolbarDefinition.node.items.movedownnode.enabled", buttonStates.canMoveDown);
    xopus_globs.set("toolbarDefinition.node.items.deletenode.enabled",   buttonStates.canDelete);
    xopus_globs.set("toolbarDefinition.node.items.insertnode.enabled",   buttonStates.canCreate);

    // Record undo/redo state.
    if (!context) context = Context.getPrevious();
    if (context) {
      canUndo = context.getXopus().canUndo();
      canRedo = context.getXopus().canRedo();
    }
    
    // (De)activate undo/redo
    xopus_globs.set("toolbarDefinition.undo.items.undo.enabled", canUndo);
    xopus_globs.set("toolbarDefinition.undo.items.redo.enabled", canRedo);
    
    // Set node inserting buttons
    for (var i in buttonStates.items)
      xopus_globs.set(buttonStates.items[i].name, buttonStates.items[i].value);        
  },
  
  doScroll:function(evt, theWindow) {
    var ta = theWindow.document.getElementById('xopus_textarea');
    if (!ta) return;
    ta.style.top = (theWindow.document.documentElement.scrollTop + theWindow.document.body.scrollTop + 10) + 'px';
  },
  
  fixSelection:function(theWindow) {
    // select the contents of a control range if it is selected
    if (this.busySelecting) return;
try {
    var rng = theWindow.document.selection.createRange();
} catch(e) { return null; }
    if (theWindow.document.selection.type == "Control")
    {
      var rng2 = this.fixRangeType(rng, theWindow.document);
      this.busySelecting=true;
      try{rng2.select();}catch(e){}
      this.busySelecting=false;
      return rng2;
    } else return rng;
  },
  fixRangeType:function(rng, theDoc) {
    // convert a control range to a text range
    var theEl = rng.item(0);
    var rng2 = theDoc.body.createTextRange();
    rng2.moveToElementText(theEl);
    return rng2;
  },
  
  storeSelection:function() {
    // Store the current selection
    if (Context.hasCurrent()) {
      var context = Context.getCurrent();
      var rng = context.xopus.htmldocument.selection.createRange();

      //see if this is a controlrange, if so: leave! run!!!^H^H^H^H^H^Hfix it
      if (rng.length) rng=fixRangeType(rng, context.xopus.htmldocument);

      var ed = context.getEditor();
      if (!xopus_globs.utils.isAncestorOf(ed, rng.parentElement()))
        return;
      
      var rng2 = rng.duplicate();
      try
      {
        rng2.moveToElementText(ed);
      }
      catch (e)
      {
//TODO!!! SOLVE THIS! WILL ERR WHEN INSERTING A NODE BEFORE THE FIRST NODE
//        __xopus_alert(e.message + "\n" + (context.xopus.htmldocument.getElementById(ed.id) == ed) + ', ' + ed.outerHTML + '\n\n' + rng2 + '\n' + rng2.text);
      }
      rng2.setEndPoint("EndToStart", rng);
      
      var noSpaceCharsInside = rng.text.replace(/\s|\u00A0/g,'');
      
      this.selObj = {
        textBeforeLen: rng2.text.length,
        textInsideLen: rng.text.length,
        noSpaceCharsBeforeLen: rng2.text.replace(/\s|\u00A0/g,'').length,
        noSpaceCharsInsideLen: noSpaceCharsInside.length,
        firstNoSpaceChar: noSpaceCharsInside.charAt(0)
      };
      
      return this.selObj;
      
      /*
      __xopus_alert("text.len:      " + rng.text.length + "\n" +
            "text.-n:       " + rng.text.replace(/\n/g, "").length + "\n" +
            "text.n:        " + rng.text.replace(/[^\n]/g, "").length + "\n" +
            "text.r:        " + rng.text.replace(/[^\r]/g, "").length + "\n" +
            "htmlText.n:    " + rng.htmlText.replace(/<[^>]*>/g, "").replace(/[^\n]/g, "").length + "\n" + 
            "htmlText.r:    " + rng.htmlText.replace(/<[^>]*>/g, "").replace(/[^\r]/g, "").length + "\n" + 
            "htmlText.len:  " + len1 + "\n" + 
            "htmlText.nbsp: " + rng.htmlText.replace(/\&nbsp;/g, "\u00A0").replace(/[^\u00A0]/g, "").length + "\n" + 
            "\n\n" + 
            "text.len:      " + rng2.text.length + "\n" +
            "text.-n:       " + rng2.text.replace(/\n/g, "").length + "\n" +
            "text.n:        " + rng2.text.replace(/[^\n]/g, "").length + "\n" + 
            "text.r:        " + rng2.text.replace(/[^\r]/g, "").length + "\n" + 
            "htmlText.n:    " + rng2.htmlText.replace(/<[^>]*>/g, "").replace(/[^\n]/g, "").length + "\n" + 
            "htmlText.r:    " + rng2.htmlText.replace(/<[^>]*>/g, "").replace(/[^\r]/g, "").length + "\n" + 
            "htmlText.len:  " + len2 + "\n" + 
            "htmlText.nbsp: " + rng2.htmlText.replace(/\&nbsp;/g, "\u00A0").replace(/[^\u00A0]/g, "").length);
      */
      //this.selLen   = rng.text.replace(/\n\r/g, "\n \r").replace(/\n/g, "").length;
      //this.selStart = rng2.text.replace(/\n\r/g, "\n \r").replace(/\n/g, "").length;


      //this.selLen = xopus_globs.utils.countRangeCharacters(rng);
      //this.selStart = xopus_globs.utils.countRangeCharacters(rng2);
      
      /*
      var str1 = rng2.text;
      var str2 = rng2.text.replace(/\n\r/g, "\n \r");
      var str3 = rng2.text.replace(/\n\r/g, "\n \r").replace(/\n/g, "");
      
      var len = Math.max(str1.length, str2.length, str3.length);
      
      var str = "";
      for (var i = 0; i < len; i++)
      {
        var c1 = "none";
        var c2 = "none";
        var c3 = "none";

        if (i < str1.length)
          c1 = str1.charCodeAt(i);

        if (i < str2.length)
          c2 = str2.charCodeAt(i);

        if (i < str3.length)
          c3 = str3.charCodeAt(i);

        str += i + ": " + c1 + " " + c2 + " " + c3 + (i == this.selStart || i == this.selStart + this.selLen ? "-" : "") + "\n";
      }
      __xopus_alert(str);
      */
      //this.selText = rng2.text.replace(/\n\r/g, "\n \r") + rng.text.replace(/\n\r/g, "\n \r");
    }
  },
  hideSelection:function() {
    //store the current selection
    this.storeSelection();
    
    //see if there is a context, if not leave
    var context = Context.getContext();
    if (!context) return;

    //hide the selection in IE
    if (xopus_globs.browser.ie) context.xopus.htmldocument.selection.empty();

    //move the cursor to the hidden textarea, this is the best way to make sure focus is hidden
    //try/catch this since it is known to fail...
    try {
      context.xopus.textarea.focus();
    } catch(e) {}
  },
  restoreSelection:function(selObj) {
    var rng = null;
    if (Context.hasCurrent()) {
      var context = Context.getCurrent();
      var editor = context.getEditor();

      //Mozilla: force the focus to go to the appropriate window by focussing a textarea in that window
      if (xopus_globs.browser.moz) {
        context.xopus.textarea.focus();
        context.xopus.textarea.blur();
      }
      
      selObj = selObj || this.selObj;
      if (!selObj) return false;
      
      var noSpaceCharsBeforeLen = selObj.noSpaceCharsBeforeLen;
      var noSpaceCharsInsideLen = selObj.noSpaceCharsInsideLen;
      var textInsideLen = selObj.textInsideLen;
      
      var rng = xopus_globs.utils.createTextRange(editor, 0, 0);
      
      // match the word chars before the selection
      rng.moveEnd("character", selObj.textBeforeLen);
      var oldDiff=0;
      while (true)
      {
        var diff = noSpaceCharsBeforeLen - rng.text.replace(/\s|\u00A0/g,'').length;
        if (!diff||diff==oldDiff) break;
        rng.moveEnd("character", diff);
        oldDiff=diff;
      }
      rng.collapse(false);
      
      if (textInsideLen)
      {
        rng.findText(selObj.firstNoSpaceChar);
        rng.collapse();
        // rng has now the correct start point

        // match the word chars inside the selection
        rng.moveEnd("character", textInsideLen);
        var oldDiff=0;
        while (true)
        {
          var diff = noSpaceCharsInsideLen - rng.text.replace(/\s|\u00A0/g,'').length;
          if (!diff||diff==oldDiff) break;
          rng.moveEnd("character", diff);
          oldDiff=diff;
        }

      }

      rng.select();

      //rng = xopus_globs.utils.createTextRange(newElt, this.selStart, this.selStart + this.selLen+1);
      //rng.select();
      
      /*
      var str = "";
      for (var i = 0; i < newElt.innerText.length; i++)
      {
        var c1 = "none";
        var c2 = "none";

        if (i < newElt.innerText.length)
          c1 = newElt.innerText.charCodeAt(i);

        if (i < this.selText.length)
          c2 = this.selText.charCodeAt(i);

        str += i + ": " + c1 + " " + c2 + (i == this.selStart || i == this.selStart + this.selLen ? "-" : "") + "\n";
      }
      __xopus_alert(str);
      */
    } else {
      //no selection: hide the cursor
      this.hideSelection();
    }

    return rng;
  },
  
  setMenuAlignment:function(menu) {
    //determine preferred alignment for the menu
    switch (xopus_globs.toolbar.curPosi) {
      case 0: menu.alignment = xopus_consts.BOTTOMLEFT; break;
      case 1: menu.alignment = xopus_consts.LEFTTOP; break;
      case 2: menu.alignment = xopus_consts.TOPLEFT; break;
      case 3: menu.alignment = xopus_consts.RIGHTTOP; break;
    }
  },
  flashXopiBorders:function(flashAll) {
    var canvasses = [];
    if (flashAll) {
      for (var id in xopus_globs.xopi) {
        var xopus = xopus_globs.xopi[id];
        if (xopus.hasChanged) {
          var canvas = xopus.canvas;
          canvasses[canvasses.length] = canvas;
        }
      }
    } else {
      var context = Context.getCurrent();
      if (!context) context = Context.getPrevious();
      if (context) canvasses[0] = context.xopus.canvas;
    }
    
    if (canvasses.length > 0) xopus_globs.animator.start([{ els:canvasses, anims:{ prop:['borderColor'], target:['#FF0000', '#00FF00'], startCode:['el.style.margin="0px"; el.style.border="dotted 1px";'], endCode:['el.style.margin="1px"; el.style.border="none";'], delay:[0], repeat:[5], duration:[5000] }}]);
  },
  showXopiBorders:function(showAll) {
    var canvasses = [];
    if (showAll) {
      for (var id in xopus_globs.xopi) {
        var xopus = xopus_globs.xopi[id];
        if (xopus.hasChanged) {
          var canvas = xopus.canvas;
          canvasses[canvasses.length] = canvas;
        }
      }
    } else {
      var context = Context.getCurrent();
      if (!context) context = Context.getPrevious();
      if (context) canvasses[0] = context.xopus.canvas;
    }
    
    for (var i=0; i<canvasses.length; i++) {
      var canvas = canvasses[i];
      canvas.style.padding = "0px";
      canvas.style.border = "dotted 1px green";
    }
  },
  hideXopiBorders:function(hideAll) {
    var canvasses = [];
    if (hideAll) {
      for (var id in xopus_globs.xopi) 
      {
        var xopus = xopus_globs.xopi[id];
        var canvas = xopus.canvas;
        canvasses[canvasses.length] = canvas;
      }
    } else {
      var context = Context.getCurrent();
      if (!context) context = Context.getPrevious();
      if (context) canvasses[0] = context.xopus.canvas;
    }
    
    for (var i=0; i<canvasses.length; i++) {
      var canvas = canvasses[i];
      canvas.style.padding = "1px";
      canvas.style.border = "none";
    }
  }
};

numScriptsLoading--;
